
package TimeTravel
{
  import TimeTravel.Player;
  import TimeTravel.MazeSquare;
  import TimeTravel.Sprites;
  import flash.display.*;
  
  public class DropObjectEvent implements WorldEvent, PlayerEvent
  {
    var at:MazeSquare;
    var playerLoc:MazeSquare;
    var objectType:String;
    var player:Player;
    var linkedWorldEvents:Array = new Array();
    var sprite:FilteredSprite;
    var dx:int, dy:int;

    public function DropObjectEvent(at:MazeSquare, player:Player, objectType:String)
    {
      this.at = at;
      this.objectType = objectType;
      this.player = player;
      this.playerLoc = player.location;
      dx = playerLoc.xcoord - at.xcoord;
      dy = playerLoc.ycoord - at.ycoord;
      linkWorldEvent(this);

      if (objectType == MazeSquare.KEY || objectType == MazeSquare.STARS || objectType == MazeSquare.LIGHT_BLOCK)
      {
	sprite = Sprites.createNamedSprite(objectType, at.maze.displayTileContentSize);
      }
    }

    public function linkWorldEvent(event:WorldEvent)
    {
      linkedWorldEvents.push(event);
    }

    public function getLinkedWorldEvents():Array
    {
      return linkedWorldEvents.slice();
    }

    public function beginPlayerForward(player:Player)
    {
      player.inventory[objectType] -= 1;
      SoundAssets.playSound(objectType==MazeSquare.STARS ? "glass-click" : "metal-hit", 0.5);
    }
    public function showPlayerForward(player:Player, completion:Number) { }
    public function endPlayerForward(player:Player) { }
    public function beginPlayerReverse(player:Player) { }
    public function showPlayerReverse(player:Player, completion:Number) { }
    public function endPlayerReverse(player:Player)
    {
      player.inventory[objectType] += 1;
      SoundAssets.playSound(objectType==MazeSquare.STARS ? "glass-click" : "metal-hit", 0.5);
    }

    private function showWorld(completion:Number)
    {
      at.setShift(dx*(1.0-completion), dy*(1.0-completion));
      at.setBlur(int(16.0*(1.0-completion)));
    }

    public function testWorldForward(maze:Maze):Array
    {
       if (at.contents != MazeSquare.EMPTY)
       {
          return [playerLoc, at, "drop"];
       }
       return null;
    }

    public function beginWorldForward(maze:Maze)
    {
      at.contents = objectType;
      at.contentsSprite = sprite;
      player.stage.addChild(sprite);
      showWorld(0.0);
    }
    
    public function showWorldForward(maze:Maze, completion:Number) { showWorld(completion); }
    public function endWorldForward(maze:Maze) { showWorld(1.0); }
    public function beginWorldReverse(maze:Maze) { showWorld(1.0); }
    public function showWorldReverse(maze:Maze, completion:Number) { showWorld(completion); }

    public function endWorldReverse(maze:Maze)
    {
      showWorld(0.0);
      player.stage.removeChild(sprite);
      at.contentsSprite = null;
      at.contents = MazeSquare.EMPTY;
    }
  }
}

