
package TimeTravel
{
  import TimeTravel.Player;
  import TimeTravel.MazeSquare;
  import flash.filters.*;
  import flash.display.*;
  import flash.geom.*;
  import flash.ui.*;
  
  public class EnterWarpEvent implements PlayerEvent, WorldEvent
  {
    var from:MazeSquare, to:MazeSquare;    
    var dx:int, dy:int;
    var stage:Sprite;
    var sprite:FilteredSprite;
    var linkedWorldEvents:Array = new Array();
    var addedToStage:Boolean = true;
    var worldTime:int;
	
    public function EnterWarpEvent(from:MazeSquare, to:MazeSquare, worldTime:int, stage:Sprite)
    {
      this.from = from;
      this.to = to;
      this.stage = stage;
      this.worldTime = worldTime;
      dx = to.xcoord - from.xcoord;
      dy = to.ycoord - from.ycoord;
      linkWorldEvent(this);
    }

    public function linkWorldEvent(event:WorldEvent)
    {
      linkedWorldEvents.push(event);
    }

    public function getLinkedWorldEvents():Array
    {
      return linkedWorldEvents.slice();
    }

    public function beginPlayerForward(player:Player)
    {
      player.location = to;
      if (to.contents != MazeSquare.EXIT_SQUARE)
      {
	SoundAssets.beginPlayingSound("warp-wiggle", 0.25, 0, 65535, "ww1");
	SoundAssets.beginPlayingSound("warp-wiggle", 0.1, 10, 65535, "ww2");
      }
    }
    
    public function showPlayerForward(player:Player, completion:Number) { }
    public function endPlayerForward(player:Player)
    {
      player.isWarping = true;
      player.sprite = to.contentsSprite;
      // player.floatingSprite = Sprites.createNamedSprite(MazeSquare.PLAYER);
      // player.floatingSprite.x = player.maze.displayUpperLeft[0] + int(player.maze.displayTileSize * to.x);
      // player.floatingSprite.x = player.maze.displayUpperLeft[1] + int(player.maze.displayTileSize * to.y);
      // stage.addChild(player.floatingSprite);
    }

    public function beginPlayerReverse(player:Player)
    {
      trace("reversing " + this);
      player.isWarping = false;
      if (to.contentsSprite != null)
      {
	to.contentsSprite.filters = [new DropShadowFilter(4, 45)];
      }
      player.sprite = sprite;

      // stage.removeChild(player.floatingSprite);
      // player.floatingSprite = null;
    }
    public function showPlayerReverse(player:Player, completion:Number) { }
    public function endPlayerReverse(player:Player)
    {
      player.location = from;
      SoundAssets.stopPlayingSound("ww1");
      SoundAssets.stopPlayingSound("ww2");
    }

    public function testWorldForward(maze:Maze):Array
    {
      if (from.contents != MazeSquare.PLAYER || (to.contents != MazeSquare.WARP && to.contents != MazeSquare.EXIT_SQUARE))
      {
        return [from, to, "warp"];
      }
      return null;
    }

    private function showWorld(completion:Number)
    {
      from.setShift(dx*completion, dy*completion);
      from.setBlur(int(16.0*completion));
    }

    public function beginWorldForward(maze:Maze)
    {
      sprite = from.contentsSprite;
      from.contents = MazeSquare.OCCUPIED;
      showWorld(0.0);
    }

    public function showWorldForward(maze:Maze, completion:Number) { showWorld(completion); }

    public function endWorldForward(maze:Maze)
    {
      showWorld(1.0);
      stage.removeChild(sprite);
      from.contents = MazeSquare.EMPTY;
      from.contentsSprite = null;
    }

    public function beginWorldReverse(maze:Maze)
    {
      from.contents = MazeSquare.OCCUPIED;
      from.contentsSprite = sprite;
      stage.addChild(sprite);
      showWorld(1.0);
    }

    public function showWorldReverse(maze:Maze, completion:Number) { showWorld(completion); }

    public function endWorldReverse(maze:Maze)
    {
      from.contents = MazeSquare.PLAYER;
      showWorld(0.0);
    }

    public function toString():String
    {
      return "warp: " + from.xcoord + "," + from.ycoord + "-->" + to.xcoord + "," + to.ycoord;
    }
  }
}

