
package TimeTravel
{
  import TimeTravel.Levels.*;
  
  public class LevelManager
  {
    private var levels:Array;
    private var levelsComplete:Array;
    private var levelsAvailable:Array;
    private var levelSelector:LevelSelector;
    
    private static var instance:LevelManager = null;

    public static var backstoryViewed:Boolean = false;
    public static var warpHintViewed:Boolean = false;
    public static var closedDoorHintViewed:Boolean = false;
    public static var eyeParadoxHintViewed:Boolean = false;
    public static var moveParadoxHintViewed:Boolean = false;
    public static var starsHintViewed:Boolean = false;
    public static var keyHintViewed:Boolean = false;
    public static var lightBlockHintViewed:Boolean = false;
    
    public static function reset()
    {
      instance = null;
      
      backstoryViewed = false;
      warpHintViewed = false;
      closedDoorHintViewed = false;
      eyeParadoxHintViewed = false;
      moveParadoxHintViewed = false;
      starsHintViewed = false;
      keyHintViewed = false;
      lightBlockHintViewed = false;
    }
    
    public static function getInstance():LevelManager
    {
      if (instance == null) {
	instance = new LevelManager();
      }
      return instance;
    }

    public function setLevelSelector(levelSelector:LevelSelector)
    {
      this.levelSelector = levelSelector;
      for each (var levelInfo:LevelInfo in levels)
      {
	levelInfo.setLevelSelector(levelSelector);
      }
    }
    
    private function addLevel(levelInfo:LevelInfo)
    {
      levelInfo.setLevelSelector(levelSelector);
      levels.push(levelInfo);
      levelsComplete.push(0);
      levelsAvailable.push(1);
    }
    
    public function LevelManager()
    {
      levels = [];
      levelsComplete = [];
      levelsAvailable = [];
      
      addLevel(new LevelRaceClockOne());
      addLevel(new LevelRaceClock());
      setLevelAvailable(2, false);
      addLevel(new LevelGetTheStars());
      setLevelAvailable(3, false);
      addLevel(new LevelLinked());
      setLevelAvailable(4, false);
      addLevel(new LevelTwoKeys());
      setLevelAvailable(5, false);
      addLevel(new LevelTroubledHub());
      setLevelAvailable(6, false);

      /*
      for (var i:int=7; i<=10; ++i) {
	addLevel(new LevelRandom("Random " + i, "medium", 9, 6));
	setLevelAvailable(i, false);
      }
      */
    }
    
    public function numLevels():int {
      return levels.length;
    }
    
    public function getLevel(ix:int):LevelInfo {
      levels[ix-1].levelNumber = ix;
      return levels[ix-1];
    }

    public function isLevelComplete(ix:int):Boolean {
      return (levelsComplete[ix-1] > 0);
    }

    public function isLevelNewlyComplete(ix:int):Boolean {
      return (levelsComplete[ix-1] == 1);
    }

    public function setLevelComplete(ix:int, b:Boolean) {
      if (b) levelsComplete[ix-1] += 1;
      else levelsComplete[ix-1] = 0;

      if (isLevelComplete(1)) setLevelAvailable(2, true);
      if (isLevelComplete(2)) { setLevelAvailable(3, true); setLevelAvailable(4, true); }
      if (isLevelComplete(3) || isLevelComplete(4)) setLevelAvailable(5, true);
      if (isLevelComplete(3) && isLevelComplete(4) && isLevelComplete(5)) setLevelAvailable(6, true);

      if (isLevelNewlyComplete(6))
      {
	addLevel(new LevelRandom("Small Random", "large", 7, 5));
	setLevelAvailable(7, true);
	addLevel(new LevelRandom("Medium Random", "medium", 9, 6));
	setLevelAvailable(8, true);
	addLevel(new LevelRandom("Large Random", "small", 13, 9));
	setLevelAvailable(9, true);
      }
    }

    public function isLevelAvailable(ix:int):Boolean {
      return (levelsAvailable[ix-1] > 0);
    }

    public function setLevelAvailable(ix:int, b:Boolean) {
      if (b) levelsAvailable[ix-1] += 1;
      else levelsAvailable[ix-1] = 0;
    }
  }
}

