
package TimeTravel.Levels
{
  import TimeTravel.*;

  public class LevelRandom extends LevelInfo
  {
    private var density:Number;
    private var objDensity:Number;
    private var mazeSize:String;
    
    public function LevelRandom(name:String, mazeSize:String, width:int, height:int, density:Number = 0.75, objDensity:Number = 0.05)
    {
      super(name, width, height);
    
      this.mazeSize = mazeSize;
      this.density = density;
      this.objDensity = objDensity;
    }
    
    public override function populateMaze(maze:Maze):Boolean
    {
      var width:int = getLevelWidth();
      var height:int = getLevelHeight();
      maze.setDisplaySize(mazeSize);
      
      for (var i:int=-1; i<=width; ++i)
      {
	for (var j:int=-1; j<=height; ++j)
	{
	  if (i>=0 && j>=0 && i<width && j<height && Math.random() < density)
	  {
	    maze.addSquare(i, j);
	  }
	}
      }

      var locs:Array = maze.getAllLocations();
      var msq:MazeSquare;
      
      for each (msq in locs)
      {
        if (Math.random() < objDensity)
        {
          maze.setContents(msq.xcoord, msq.ycoord, MazeSquare.BLOCK);
        }
        else if (Math.random() < objDensity)
        {
          maze.setContents(msq.xcoord, msq.ycoord, MazeSquare.LIGHT_BLOCK);
        }
        else if (Math.random() < objDensity)
        {
          maze.setContents(msq.xcoord, msq.ycoord, MazeSquare.WARP);
        }
        else if (Math.random() < objDensity)
        {
          maze.setContents(msq.xcoord, msq.ycoord, MazeSquare.STARS);
        }
        else if (Math.random() < objDensity)
        {
          maze.setContents(msq.xcoord, msq.ycoord, MazeSquare.KEY);
        }
      }

      for each (msq in locs)
      {
        if (msq.contents == MazeSquare.EMPTY && msq.hasNeighbor(1, 0) && msq.getNeighbor(1, 0).contents == MazeSquare.EMPTY && !msq.hasNeighbor(-1, 0))
        {
          maze.setEntrance(msq.xcoord, msq.ycoord, 1, 0);
	  break;
        }
      }

      for each (msq in locs)
      {
	if (msq.contents == MazeSquare.EMPTY && msq.hasNeighbor(-1, 0) && msq.getNeighbor(-1, 0).contents == MazeSquare.EMPTY && !msq.hasNeighbor(1, 0))
	{
	  maze.setExit(msq.xcoord, msq.ycoord, 1, 0);
	  break;
	}
      }
    
      return true;
    }
  }
}

