
package TimeTravel
{
  import TimeTravel.MazeSquare;
  import TimeTravel.EventHistory;
  import flash.display.*;
  
  public class Maze
  {
    var allLocs:Array;
    var loc:Array;
    var width:int, height:int;
    public var entranceSquare:MazeSquare;
    var n:int = 0;

    public var currentTime:int = 0;
    var currentTimeExtra:int = 0;
    var FPS:int = 24;
    
    var eventHistory:EventHistory = new EventHistory();

    public var displayUpperLeft:Array = [64, 64];
    public var displayTileSize:int = 54;
    public var displayTileContentSize:int = 48;

    public var currentParadox:Array = null;
    public var paradoxSprite:FilteredSprite = null;

    public var recordedSight:Array = [];

    public function getLastWorldEventTime():int
    {
      return eventHistory.getLastEventTime();
    }

    public function setDisplaySize(size:String)
    {
      displayUpperLeft = [64, 64];
      if (size == "large")
      {
	displayTileSize = 52;
	displayTileContentSize = 48;
      }
      else if (size == "medium")
      {
	displayTileSize = 39;
	displayTileContentSize = 36;
      }
      else if (size == "small")
      {
	displayTileSize = 26;
	displayTileContentSize = 24;
      }
    }
    
    public function getAllLocations():Array
    {
      return allLocs.slice();
    }

    public function hasLocation(x:int, y:int):Boolean
    {
      return x>=0 && x<width && y>=0 && y<height && (loc[x][y] !== false);
    }

    public function getLocation(x:int, y:int):MazeSquare
    {
      return this.loc[x][y];
    }

    public function setExit(x:int, y:int, dx:int, dy:int)
    {
      this.loc[x][y].contents = MazeSquare.EXIT_SQUARE;
      this.loc[x][y].xdir = dx;
      this.loc[x][y].ydir = dy;

      this.loc[x][y].setVisible(true);
      this.loc[x][y].maskSprite = null;
    }

    public function setEntrance(x:int, y:int, dx:int, dy:int)
    {
      this.loc[x][y].contents = MazeSquare.ENTRANCE_SQUARE;
      this.loc[x][y].xdir = dx;
      this.loc[x][y].ydir = dy;
      this.entranceSquare = this.loc[x][y];

      this.loc[x][y].setVisible(true);
      this.loc[x][y].maskSprite = null;
    }

    public function setContents(x:int, y:int, objectType:String)
    {
      var msq:MazeSquare = this.loc[x][y];
      msq.contents = objectType;
      msq.xshift = msq.yshift = 0.0;

      if (objectType == MazeSquare.DOOR)
      {
	var doorType:String = "door";
	if (msq.keysNeeded >= 1 && msq.keysNeeded <= 3) {
	  doorType += "-" + msq.keysNeeded + "-key";
	}
	msq.contentsSprite = Sprites.createNamedSprite(doorType, displayTileContentSize);
	msq.doorSprite = msq.contentsSprite;
      }
      else
      {
	msq.contentsSprite = Sprites.createNamedSprite(objectType, displayTileContentSize);
      }
      
      msq.update();
    }

    public function addSquare(x:int, y:int)
    {
      if (this.loc[x][y] != false) { return; }
      this.loc[x][y] = new MazeSquare(this, x, y);
      this.allLocs[n++] = this.loc[x][y];
    }

    public function linkBlocks(x0:int, y0:int, x1:int, y1:int)
    {
      this.loc[x0][y0].blockIsLinked = [x1-x0, y1-y0];
      this.loc[x1][y1].blockIsLinked = [x0-x1, y0-y1];
      this.loc[x0][y0].paintLink(this.loc[x0][y0].contentsSprite.graphics, 0x0000FF, x1, y1);
    }

    public function Maze()
    {
    }
    
    public function initDimensions(width:int, height:int)
    {
      this.width = width;
      this.height = height;
      this.loc = new Array();
      this.allLocs = new Array();
      for (var i:int=-1; i<=width; ++i)
      {
	this.loc[i] = new Array();
	for (var j:int=-1; j<=height; ++j)
	{
	  this.loc[i][j] = false;
	}
      }
    }
    
    public static function createRandomMaze(width:int, height:int, density:Number):Maze
    {
      var maze:Maze = new Maze();
      maze.initDimensions(width, height);
      for (var i:int=-1; i<=width; ++i)
      {
	for (var j:int=-1; j<=height; ++j)
	{
	  if (i>=0 && j>=0 && i<width && j<height && Math.random() < density)
	  {
	    maze.addSquare(i, j);
	  }
	}
      }
      
      return maze;
    }

    public static function createMazeFromLevelInfo(levelInfo:LevelInfo):Maze
    {
      var maze:Maze = new Maze();
      maze.initDimensions(levelInfo.getLevelWidth(), levelInfo.getLevelHeight());
      var ok:Boolean = levelInfo.populateMaze(maze);
      if (!ok)
      {
	maze = createRandomMaze(levelInfo.getLevelWidth(), levelInfo.getLevelHeight(), 0.75);
      }
      return maze;
    }

    private function isVisiblePath(x0:Number, y0:Number, x1:Number, y1:Number, eps:Number = 0.0)
    {
      var dx:Number = x1 - x0;
      var dy:Number = y1 - y0;
      var L:Number = 0.0;
      
      if (!hasLocation(int(x1), int(y1))) return false;
      if (int(x1)==int(x0) && int(y1)==int(y0)) return true;
      var msqFinal:MazeSquare = getLocation(int(x1), int(y1));
      var msq:MazeSquare;
      
      while (L < 1.0)
      {
	if (!hasLocation(int(L*dx+x0), int(L*dy+y0))) return false;
	
	if (eps > 0.0) for (var ex:int = -1; ex <= 1; ex += 2) for (var ey:int = -1; ey <= 1; ey += 2)
	{
	  if (!hasLocation(int(L*dx+x0+ex*eps), int(L*dy+y0+ey*eps))) return false;
	  msq = getLocation(int(L*dx+x0+ex*eps), int(L*dy+y0+ey*eps));
	  if (msq != msqFinal && msq.isOpaque()) { return false; }
	}
	
	msq = getLocation(int(L*dx+x0), int(L*dy+y0));
	if (msq == msqFinal) return true;
	if (msq.isOpaque()) { return false; }
		
	var ydist:Number = -1.0;
	var xdist:Number = -1.0;
	
	if (y1>y0)
	{
	  ydist = (Math.ceil(L*dy+y0) - (L*dy+y0)) / dy;
	}
	else if (y1<y0)
	{
	  ydist = ((L*dy+y0) - Math.floor(L*dy+y0)) / (-dy);
	}

	if (x1>x0)
	{
	  xdist = (Math.ceil(L*dx+x0) - (L*dx+x0)) / dx;
	}
	else if (x1<x0)
	{
	  xdist = ((L*dx+x0) - Math.floor(L*dx+x0)) / (-dx);
	}
	
	if ((ydist <= xdist && ydist > 0.0) || xdist < 0.0)
	{
	  L += (ydist + 0.001);
	}
	else
	{
	  L += (xdist + 0.001);
	}
      }
      
      return true;
    }
    
    public function getVisibleSquares(msq:MazeSquare):Array
    {
      var ret:Array = new Array();

      var buffer:Array = new Array();
      buffer.push(msq);
      
      while (buffer.length > 0)
      {
	var nextBuffer:Array = new Array();
	for each (var msq2:MazeSquare in buffer)
	{
	  if (isVisiblePath(msq.x+0.5, msq.y+0.5, msq2.x+0.5, msq2.y+0.5, 0.01))
	  {
	    ret.push(msq2);
	    if (!msq2.isOpaque()) for (var dx:int = -1; dx <= 1; ++dx) for (var dy:int = -1; dy <= 1; ++dy)
	    {
	      if (msq2.hasNeighbor(dx, dy) && nextBuffer.indexOf(msq2.getNeighbor(dx, dy)) < 0)
	      {
		nextBuffer.push(msq2.getNeighbor(dx, dy));
	      }
	    }
	  }
	}
	for each (msq2 in ret)
	{
	  var id:int = nextBuffer.indexOf(msq2);
	  if (id >= 0) nextBuffer.splice(id, 1);
	}
	for each (msq2 in buffer)
	{
	  var id:int = nextBuffer.indexOf(msq2);
	  if (id >= 0) nextBuffer.splice(id, 1);
	}
	buffer = nextBuffer;
      }

      return ret;
    }
    
    public function advanceWorldTime(amount:int):Boolean
    {
      var event:WorldEvent;
      var paradox:Array = [];
      if (amount <= 0) { return true; }
        
      // if starting new time step, begin events...
      // check for paradox before beginning each event.
      // if paradox is detected, return to the breakpoint.
      if (currentTimeExtra == 0)
      {
	for each (event in eventHistory.getEvents(currentTime))
	{
	  paradox = event.testWorldForward(this);
	  
	  if (paradox != null)
	  {
	    trace("paradox!!! squares " + paradox[0] + " and " + paradox[1]);
	    var startErasing:Boolean = false;
	    var event2:WorldEvent;
	    for each (event2 in eventHistory.getReversedEvents(currentTime))
	    {
	      if (startErasing) event2.endWorldReverse(this);
	      if (event2 == event) startErasing = true;
	    }
	    
	    break;
	  }
	  
	  event.beginWorldForward(this);
	}
      }
      
      if (paradox != null && paradox.length >= 2)
      {
	setParadox(paradox);
	return false;
      }
             
      currentTimeExtra = Math.min(currentTimeExtra + amount, FPS);
              
      // show all events in intermediate state
      for each (event in eventHistory.getEvents(currentTime))
      {
	event.showWorldForward(this, currentTimeExtra / (1.0*FPS));
      }
       
      // if we've moved all the way to the next breakpoint, end
      // all events
      if (currentTimeExtra == FPS)
      {
	for each (event in eventHistory.getEvents(currentTime))
	{
	  event.endWorldForward(this);
	}
	++currentTime;
	currentTimeExtra = 0;
      }

      return true;
    }

    public function addEvent(event:WorldEvent, prepend:Boolean = false)
    {
      trace("adding world event to time " + currentTime);
      if (prepend) eventHistory.prependEvent(currentTime, event);
      else eventHistory.addEvent(currentTime, event);
    }

    public function addEventToTime(event:WorldEvent, t:int, prepend:Boolean = false)
    {
      if (prepend) eventHistory.prependEvent(t, event);
      else eventHistory.addEvent(t, event);
    }
    
    public function setParadox(paradox:Array)
    {
      var maze:Maze = this;
      var xc1:int = maze.displayUpperLeft[0] + MazeSquare(paradox[0]).xcoord * maze.displayTileSize;
      var yc1:int = maze.displayUpperLeft[1] + MazeSquare(paradox[0]).ycoord * maze.displayTileSize;
      var xc2:int = maze.displayUpperLeft[0] + MazeSquare(paradox[1]).xcoord * maze.displayTileSize;
      var yc2:int = maze.displayUpperLeft[1] + MazeSquare(paradox[1]).ycoord * maze.displayTileSize;
      var xw:int = maze.displayTileContentSize / 2;
      var yw:int = maze.displayTileContentSize / 2;
      
      currentParadox = paradox.slice();
      
      paradoxSprite = new FilteredSprite();
      paradoxSprite.graphics.lineStyle(4, 0xFF0000);
      if (paradox.length == 2 || paradox[2] != "eye")
      {
	paradoxSprite.graphics.drawEllipse(xc1 - xw - 2, yc1 - xw - 2, 2*xw + 4, 2*yw + 4);
	paradoxSprite.graphics.drawEllipse(xc2 - xw - 2, yc2 - xw - 2, 2*xw + 4, 2*yw + 4);

	if (xc2!=xc1 || yc2!=yc1)
	{
	  var parrow:FilteredSprite = Sprites.createNamedSprite("red-right-arrow", maze.displayTileContentSize);
	  parrow.glowColor = "red";
	  parrow.updateFilters();
	  parrow.updateFilters();
	  
	  parrow.x = xc1;
	  parrow.y = yc1;
	  
	  if (xc2 == xc1+maze.displayTileSize) { parrow.rotation = 0; parrow.x += 0.5*maze.displayTileSize; }
	  else if (yc2 == yc1+maze.displayTileSize) { parrow.rotation = 90; parrow.y += 0.5*maze.displayTileSize; }
	  else if (xc2 == xc1-maze.displayTileSize) { parrow.rotation = 180; parrow.x -= 0.5*maze.displayTileSize; }
	  else if (yc2 == yc1-maze.displayTileSize) { parrow.rotation = 270; parrow.y -= 0.5*maze.displayTileSize; }
	  
	  paradoxSprite.addChild(parrow);
	}
      }
      else
      {
	var atTop:Boolean = (MazeSquare(paradox[0]).ycoord < 3);
	
	paradoxSprite.graphics.drawRect(xc1 - xw - 2, yc1 - xw - 2, 2*xw + 4, 2*yw + 4);
	paradoxSprite.graphics.drawEllipse(xc2 - xw - 2, yc2 - xw - 2, 2*xw + 4, 2*yw + 4);
	var pd:FilteredSprite = Sprites.createNamedSprite("paradox-dude", maze.displayTileContentSize);
	
	pd.x = xc1 + maze.displayTileContentSize / 2;
	pd.y = yc1 - maze.displayTileContentSize;
	
	if (atTop)
	{
	  pd.rotation = 180;
	  pd.x -= maze.displayTileContentSize;
	  pd.y += 2.0 * maze.displayTileContentSize;
	}
	
	var oldView:FilteredSprite;
	if (paradox[3] == MazeSquare.EMPTY)
	{
	  oldView = new FilteredSprite();
	  paradox[1].paint(oldView.graphics, 0xCCCCCC, true);
	  oldView.width *= 0.75;
	  oldView.height *= 0.75;
	}
	else
	{
	  oldView = Sprites.createNamedSprite(paradox[3], maze.displayTileContentSize);
	}
	
	if (atTop)
	{
	  oldView.rotation = 180;
	}
	
	oldView.fade = 0.75;
	oldView.updateFilters();
	pd.addChild(oldView);
	oldView.x += maze.displayTileContentSize / 10;
	oldView.y -= maze.displayTileContentSize / 2;
	paradoxSprite.addChild(pd);
      }
      
      var dx:int = xc2 - xc1;
      var dy:int = yc2 - yc1;
      var dsqr:int = dx*dx + dy*dy;
      var unitx:Number = dx / (1.0*Math.sqrt(dsqr));
      var unity:Number = dy / (1.0*Math.sqrt(dsqr));
      paradoxSprite.graphics.moveTo(xc1 + int(unitx * (xw + 2)), yc1 + int(unity * (yw + 2)));
      paradoxSprite.graphics.lineTo(xc2 - int(unitx * (xw + 2)), yc2 - int(unity * (yw + 2)));
    }
    
    public function removeEvent(time:int, event:WorldEvent)
    {
      var ok:Boolean = eventHistory.removeEvent(time, event);
      if (!ok) trace("error removing event from time " + time);
    }

    public function rewindWorldTime(amount:int)
    {
      var event:WorldEvent;
      if (amount <= 0) { return; }
      
       // if starting new reversed time step, begin reversed events
       if (currentTimeExtra == 0)
       {
	 --currentTime;
	 currentTimeExtra = FPS;
	 for each (event in eventHistory.getReversedEvents(currentTime))
	 {
	   event.beginWorldReverse(this);
	 }
       }
       
       currentTimeExtra = Math.max(currentTimeExtra - amount, 0);
       
       // show all events in intermediate state
       for each (event in eventHistory.getReversedEvents(currentTime))
       {
	 event.showWorldReverse(this, currentTimeExtra / (1.0*FPS));
       }
       
       // if we've moved all the way to the next breakpoint, end
       // reversed events.
       if (currentTimeExtra == 0)
       {
	 for each (event in eventHistory.getReversedEvents(currentTime))
	 {
	   event.endWorldReverse(this);
	 }
       }
    }
  }
}

