
package TimeTravel
{
  import TimeTravel.Maze;
  import flash.display.*;
  import flash.filters.*;
  
  public class MazeSquare
  {
    var x:int, y:int;
    var maze:Maze;

    public static const EMPTY:String = "empty";
    public static const BLOCK:String = "block";
    public static const LIGHT_BLOCK:String = "light-block";
    public static const PLAYER:String = "player";
    public static const WARP:String = "warp";
    public static const DOOR:String = "door";
    public static const KEY:String = "key";
    public static const STARS:String = "stars";
    public static const OCCUPIED:String = "occupied";
    public static const EXIT_SQUARE:String = "exit-square";
    public static const ENTRANCE_SQUARE:String = "entrance-square";

    public var contents:String = EMPTY;
    public var contentsSprite:FilteredSprite = null;
    public var maskSprite:FilteredSprite = null;
    public var parentShape:Sprite = null;
    public var isVisible:Boolean = true;
    
    public var doorSprite:FilteredSprite = null;
    public var doorClosed:Boolean = false;
    public var keysNeeded:int = 0;
    
    public var xshift:Number = 0.0;
    public var yshift:Number = 0.0;
    public var blur:int = 0;
    public var xdir:int, ydir:int;
    public var blockIsLinked:Array = [];
    public var blockedEdge:Array = [];

    public var starsNeeded:int = 0;
    public var starsReceived:int = 0;
    public var starsTarget:MazeSquare = null;
    public var starsSprite:FilteredSprite;
    
    public function setParentShape(parentShape:Sprite)
    {
      this.parentShape = parentShape;
    }
    
    public function MazeSquare(maze:Maze, x:int, y:int)
    {
      this.maze = maze;
      this.x = x;
      this.y = y;
    }

    public function hasNeighbor(dx:int, dy:int):Boolean
    {
      if (dx*dx + dy*dy != 1) { return false; }
      return maze.hasLocation(x+dx, y+dy) && (blockedEdge == [] || blockedEdge[0] != dx || blockedEdge[1] != dy);
    }

    public function getNeighbor(dx:int, dy:int):MazeSquare
    {
      return maze.getLocation(x+dx, y+dy);
    }

    public function get xcoord() { return x; }
    public function get ycoord() { return y; }
    public function getCoords() { return [x, y]; }

    public function paintArrow(g:Graphics, dx:int, dy:int, color:uint = 0xCCCCCC)
    {
      var xc:int = maze.displayUpperLeft[0] + x * maze.displayTileSize;
      var yc:int = maze.displayUpperLeft[1] + y * maze.displayTileSize;
      var xw:int = maze.displayTileContentSize / 2;
      var yw:int = maze.displayTileContentSize / 2;

      g.beginFill(color);
      g.moveTo(xc + dx*xw, yc + dy*yw);
      g.lineTo(xc - dx*xw + dy*yw, yc - dy*yw + dx*xw);
      g.lineTo(xc - dx*xw - dy*yw, yc - dy*yw - dx*xw);
      g.lineTo(xc + dx*xw, yc + dy*yw);
      g.endFill();
    }
      
    public function paint(g:Graphics, color:uint = 0xCCCCCC, relativePosition:Boolean = false)
    {
      if (contents == ENTRANCE_SQUARE || contents == EXIT_SQUARE)
      {
         paintArrow(g, xdir, ydir, color);
         return;
      }

      var xc:int = maze.displayUpperLeft[0] + x * maze.displayTileSize;
      var yc:int = maze.displayUpperLeft[1] + y * maze.displayTileSize;
      var xw:int = maze.displayTileContentSize / 2;
      var yw:int = maze.displayTileContentSize / 2;

      if (relativePosition)
      {
	xc = 0;
	yc = 0;
      }
      
      g.beginFill(color);
      g.drawRoundRect(xc - xw, yc - yw, 2*xw, 2*yw, 8*xw/5);
      g.endFill();
      
      for (var dx:int=-1; dx<=1; dx+=2) for (var dy:int=-1; dy<=1; dy+=2)
      {
        if (hasNeighbor(dx,0) || hasNeighbor(0,dy))
        {
          g.beginFill(color);
          g.drawRect(xc + ((dx-1)/2)*xw, yc + ((dy-1)/2)*yw, xw, yw);
          g.endFill();
        }
      }
    }

    public function paintLink(g:Graphics, color:uint, x1:int, y1:int)
    {
      var linkWidth:int = maze.displayTileContentSize / 10;

      g.beginFill(color);
      // g.drawRect(-15, -5, 54*(x1-x) + 30, 10);
      if (x1!=x) g.drawRect(-3*linkWidth, -linkWidth, maze.displayTileSize * (x1-x) + 6*linkWidth, 2*linkWidth);
      else if (y1!=y) g.drawRect(-linkWidth, -3*linkWidth, 2*linkWidth, maze.displayTileSize * (y1-y) + 6*linkWidth);
      g.endFill();
    }
    
    public function setShift(x:Number, y:Number)
    {
      xshift = x;
      yshift = y;
      update();
    }

    public function setBlur(blur:int)
    {
      this.blur = blur;
      updateBlur();
    }

    public function setVisible(b:Boolean)
    {
      if (maskSprite == null) {
	maskSprite = new FilteredSprite();
	paint(maskSprite.graphics, 0x000000);
	maskSprite.fade = 0.25;
	maskSprite.updateFilters();
      }
      
      if (parentShape != null) {
	if (b == true && isVisible == false) { isVisible=true; parentShape.removeChild(maskSprite); }
	else if (b == false && isVisible == true)
	{
	  isVisible=false;
	  parentShape.addChild(maskSprite);
	}
      }
      else {
	isVisible = b;
      }
    }
    
    public function update()
    {
      if (contentsSprite != null)
      {
        contentsSprite.x = maze.displayUpperLeft[0] + int(maze.displayTileSize * (x + xshift));
        contentsSprite.y = maze.displayUpperLeft[1] + int(maze.displayTileSize * (y + yshift));
      }
      if (starsSprite != null)
      {
        starsSprite.x = maze.displayUpperLeft[0] + x * maze.displayTileSize;
        starsSprite.y = maze.displayUpperLeft[1] + y * maze.displayTileSize;
      }
      
      else trace("error... contents are null")
    }

    public function updateBlur()
    {
      if (contentsSprite != null)
      {
        if (blur > 0) contentsSprite.filters = [new BlurFilter(blur, blur, 2), new DropShadowFilter(4, 45)];
        else contentsSprite.filters = [new DropShadowFilter(4, 45)];
      }
    }

    public function isOpaque():Boolean
    {
      return (contents==MazeSquare.BLOCK || contents==MazeSquare.LIGHT_BLOCK || (contents==MazeSquare.DOOR && doorClosed));
    }
  }
}

