
package TimeTravel
{
  import flash.display.*;
  import flash.text.*;
  
  public class Player
  {
    public var sprite:FilteredSprite;
    public var floatingSprite:FilteredSprite = null;
    
    public var currentTime:int = 0;
    public var currentTimeExtra:int = 0;
    public var FPS:int = 24;

    var eventHistory:EventHistory = new EventHistory();
    public var inventory:Object = new Object();
    var inventoryDisplays:Object;

    public var location:MazeSquare;
    public var maze:Maze;
    public var isWarping:Boolean;
    public var isDropping:String = MazeSquare.EMPTY;
    public var stage:Sprite;
    public var warpLocation:MazeSquare, warpDepartureLocation:MazeSquare;
    public var warpTime:int;

    var playerInventorySprite:Object = new Object();

    public function Player(maze:Maze, inventory:Object, inventorySprite:Object, stage:Sprite)
    {
      this.location = maze.entranceSquare.getNeighbor(maze.entranceSquare.xdir, maze.entranceSquare.ydir);
      this.maze = maze;
      this.inventoryDisplays = inventory;
      this.playerInventorySprite = inventorySprite;
      this.stage = stage;

      this.inventory["light-block"] = 0;
      this.inventory["key"] = 0;
      this.inventory["stars"] = 0;

      var arrival:PlayerArrivesEvent = new PlayerArrivesEvent(maze.entranceSquare, this.location, stage, false, -1);
      // eventHistory.addEvent(-100, arrival);
      
      maze.addEvent(arrival);

      // eventHistory.addEvent(0, new PlayerWaitEvent(null));
      
      arrival.beginPlayerForward(this);
      arrival.endPlayerForward(this);

      update();
    }

    public function getNextEvent():PlayerEvent
    {
      if (eventHistory.getEvents(currentTime).length > 0) {
	return PlayerEvent(eventHistory.getEvents(currentTime)[0]);
      }
      return null;
    }

    public function getPreviousEvent():PlayerEvent
    {
      if (currentTimeExtra == 0 && eventHistory.getEvents(currentTime-1).length > 0) {
	return PlayerEvent(eventHistory.getEvents(currentTime-1)[0]);
      }
      else if (currentTimeExtra != 0 && eventHistory.getEvents(currentTime).length > 0) {
	return PlayerEvent(eventHistory.getEvents(currentTime)[0]);
      }
      return null;
    }

    function addEvent(event:PlayerEvent, prepend:Boolean = false)
    {
      if (prepend) eventHistory.prependEvent(currentTime, event);
      else eventHistory.addEvent(currentTime, event);
    }

    function addEventToTime(event:PlayerEvent, t:int, prepend:Boolean = false)
    {
      if (prepend) eventHistory.prependEvent(t, event);
      else eventHistory.addEvent(t, event);
    }
    
    public function move(dx:int, dy:int)
    {
      if (currentTimeExtra != 0)
      {
         trace("error... called doMove out of synch")
      }

      if (dx==0 && dy==0)
      {
         if (isDropping != MazeSquare.EMPTY)
         {
            isDropping = MazeSquare.EMPTY;
            return;
         }
         trace("adding wait at " + currentTime);
         newEvent = new PlayerWaitEvent(location);
         
	 addEvent(newEvent, true);
         
	 return;
      }

      if (!location.hasNeighbor(dx, dy))
      {
         return;
      }

      var neighbor:MazeSquare = location.getNeighbor(dx, dy)
      var newEvent:*;

      if (isDropping != MazeSquare.EMPTY)
      {
	 if (neighbor.contents == MazeSquare.EMPTY)
         {
	   if (isDropping == MazeSquare.STARS && neighbor.starsReceived < neighbor.starsNeeded)
	   {
	     newEvent = new PlaceStarsEvent(neighbor, this);
	     // maze.addEvent(newEvent);
	     // addEvent(newEvent);
	     if (neighbor.starsReceived == neighbor.starsNeeded - 1)
	     {
	       var triggerEvent:WorldEvent = new ObjectVanishEvent(neighbor.starsTarget, neighbor.starsTarget.contents, this);
	       maze.addEvent(triggerEvent, true);
	       newEvent.linkWorldEvent(triggerEvent);
	     }
	     maze.addEvent(newEvent, true);
	     addEvent(newEvent, true);
	   }
	   else
	   {
	     newEvent = new DropObjectEvent(neighbor, this, isDropping);
	     maze.addEvent(newEvent, true);
	     addEvent(newEvent, true);
	   }
	   isDropping = MazeSquare.EMPTY;
         }
         return;
      }
      
      if (neighbor.contents == MazeSquare.BLOCK && neighbor.hasNeighbor(dx, dy) && neighbor.getNeighbor(dx, dy).contents == MazeSquare.EMPTY)
      {
	if (neighbor.blockIsLinked.length > 0)
	{
	  var nx:int = neighbor.blockIsLinked[0] + neighbor.xcoord;
	  var ny:int = neighbor.blockIsLinked[1] + neighbor.ycoord;
	  if (maze.hasLocation(nx+dx, ny+dy) && maze.getLocation(nx+dx, ny+dy).contents == MazeSquare.EMPTY)
	  {
	    newEvent = new ObjectMoveEvent(maze.getLocation(nx, ny), maze.getLocation(nx+dx,ny+dy), maze.getLocation(nx,ny).contents);
	    var slideEvent:* = new SlideObjectEvent(neighbor, neighbor.getNeighbor(dx, dy), this, neighbor.contents);
	    slideEvent.linkWorldEvent(newEvent);
	    // maze.addEvent(newEvent);
	    // maze.addEvent(slideEvent);
	    // addEvent(slideEvent);
	    addEvent(slideEvent, true);
	    maze.addEvent(slideEvent, true);
	    maze.addEvent(newEvent, true);
	  }
	}
	else
	{
	  trace("adding move & push at " + currentTime);
	  newEvent = new SlideObjectEvent(neighbor, neighbor.getNeighbor(dx, dy), this, neighbor.contents);
	  maze.addEvent(newEvent, true);
	  addEvent(newEvent, true);
	}
      }
      else if (neighbor.contents == MazeSquare.EMPTY || (neighbor.contents == MazeSquare.DOOR && !neighbor.doorClosed))
      {
	trace("adding move at " + currentTime);
	newEvent = new PlayerMoveEvent(location, neighbor);
	// maze.addEvent(newEvent);
	// addEvent(newEvent);
	if (location.doorSprite != null && !location.doorClosed && location.keysNeeded > 0 && location.keysNeeded <= 3)
	{
	  var closeDoor:DoorCloseEvent = new DoorCloseEvent(location);
	  newEvent.linkWorldEvent(closeDoor);
	  maze.addEvent(closeDoor, true);
	}
	maze.addEvent(newEvent, true);
	addEvent(newEvent, true);
      }
      else if (neighbor.contents == MazeSquare.DOOR && neighbor.doorClosed && inventory["key"] >= neighbor.keysNeeded)
      {
	var openDoor:DoorOpenEvent = new DoorOpenEvent(neighbor);
	maze.addEvent(openDoor, true);
	addEvent(openDoor, true);
      }
      else if (neighbor.contents == MazeSquare.KEY || neighbor.contents == MazeSquare.STARS || neighbor.contents == MazeSquare.LIGHT_BLOCK)
      {
	trace("adding take-object at " + currentTime);
	var takeEvent:TakeObjectEvent = new TakeObjectEvent(neighbor, this, neighbor.contents);
	maze.addEvent(takeEvent, true);
	addEvent(takeEvent, true);
      }
      else if (neighbor.contents == MazeSquare.WARP ||
	       (neighbor.contents == MazeSquare.EXIT_SQUARE && neighbor.xdir == dx && neighbor.ydir == dy))
      {
	trace("entering warp at " + currentTime);
	var warpEvent:EnterWarpEvent = new EnterWarpEvent(location, neighbor, maze.currentTime, stage);
	maze.addEvent(warpEvent, true);
	addEvent(warpEvent, true);
	warpLocation = neighbor;
	warpDepartureLocation = location;
	warpTime = maze.currentTime;
      }
    }

    public function dropOutOfWarp()
    {
       if (!isWarping) { return; }
       trace("dropping out of warp?: time=" + currentTime + " " + currentTimeExtra + ", maze at " + maze.currentTime + " " + maze.currentTimeExtra);

       if (warpDepartureLocation.contents != MazeSquare.EMPTY)
       {
         trace("can't do it.")
         return;
       }

       // isWarping = false;
 
       var arrivalEvent:PlayerArrivesEvent = new PlayerArrivesEvent(warpLocation, warpDepartureLocation, stage, true, warpTime);
       addEvent(arrivalEvent, true);
       maze.addEvent(arrivalEvent, true);
    }

    public function advancePlayerTime(amount:int)
    {
       var event:PlayerEvent;
       if (amount <= 0) { return; }
        
       // if starting new time step, begin events
       if (currentTimeExtra == 0)
       {
	 for each (event in eventHistory.getEvents(currentTime))
	 {
	   event.beginPlayerForward(this);
	 }
       }
       
       currentTimeExtra = Math.min(currentTimeExtra + amount, FPS);
              
       // show all events in intermediate state
       for each (event in eventHistory.getEvents(currentTime))
       {
	 event.showPlayerForward(this, currentTimeExtra / (1.0*FPS));
       }
       
       // if we've moved all the way to the next breakpoint, end
       // all events
       if (currentTimeExtra == FPS)
       {
	 for each (event in eventHistory.getEvents(currentTime))
	 {
	   event.endPlayerForward(this);
	 }
	 ++currentTime;
	 currentTimeExtra = 0;
       }
    }
    
    public function rewindPlayerTime(amount:int)
    {
       var event:PlayerEvent;
       if (amount <= 0) { return; }

       // if starting new reversed time step, begin reversed events
       if (currentTimeExtra == 0)
       {
	 --currentTime;
	 currentTimeExtra = FPS;
	 for each (event in eventHistory.getReversedEvents(currentTime))
	 {
	   event.beginPlayerReverse(this);
	 }
       }
       
       currentTimeExtra = Math.max(currentTimeExtra - amount, 0);
       
       // show all events in intermediate state
       for each (event in eventHistory.getReversedEvents(currentTime))
       {
	 event.showPlayerReverse(this, currentTimeExtra / (1.0*FPS));
       }
       
       // if we've moved all the way to the next breakpoint, end
       // reversed events, delete them, and delete associated world
       // events.
       if (currentTimeExtra == 0)
       {
	 removeEventsFromPlayerTimeline(true);
       }
    }

    public function removeEventsFromPlayerTimeline(eventsAreBegun:Boolean = false)
    {
      for each (var event:PlayerEvent in eventHistory.getReversedEvents(currentTime))
      {
	for each (var worldEvent:WorldEvent in event.getLinkedWorldEvents())
	{
	  maze.removeEvent(maze.currentTime, worldEvent);
	}
	if (eventsAreBegun) event.endPlayerReverse(this);
	eventHistory.removeEvent(currentTime, event);
      }
    }

    public function update()
    {
       for (var name:String in inventory)
       {
         var n:int = inventory[name];
         if (n!=0)
         {
           inventoryDisplays[name].text = "" + n;
           FilteredSprite(playerInventorySprite[name]).fade = 1.0;
	   FilteredSprite(playerInventorySprite[name]).updateFilters();
         }
         else
         {
           inventoryDisplays[name].text = "";
           FilteredSprite(playerInventorySprite[name]).fade = 0.25;
	   FilteredSprite(playerInventorySprite[name]).updateFilters();
         }
       }
    }
  }
}
