
package TimeTravel
{
  import TimeTravel.Player;
  import TimeTravel.MazeSquare;
  import flash.filters.*;
  import flash.display.*;
  import flash.geom.*;
  import flash.ui.*;
  
  public class PlayerArrivesEvent implements PlayerEvent, WorldEvent
  {
    var from:MazeSquare, to:MazeSquare;    
    var linkedWorldEvents:Array = new Array();
    var stage:Sprite;
    var addedToStage:Boolean;
    var fromWarp:Boolean = false;
    var warpTime:int = -1;
    var sprite:FilteredSprite;
    var dx:int, dy:int;
    
    public function PlayerArrivesEvent(from:MazeSquare, to:MazeSquare, stage:Sprite, fromWarp:Boolean, warpTime:int)
    {
      this.from = from;
      this.to = to;
      this.stage = stage;
      dx = to.xcoord - from.xcoord;
      dy = to.ycoord - from.ycoord;
      this.fromWarp = fromWarp;
      this.warpTime = warpTime;
      linkWorldEvent(this);

      sprite = Sprites.createNamedSprite(MazeSquare.PLAYER, from.maze.displayTileContentSize);
    }

    public function linkWorldEvent(event:WorldEvent)
    {
      linkedWorldEvents.push(event);
    }

    public function getLinkedWorldEvents():Array
    {
      return linkedWorldEvents.slice();
    }

    public function beginPlayerForward(player:Player)
    {
      player.isWarping = false;
      SoundAssets.stopPlayingSound("ww1");
      SoundAssets.stopPlayingSound("ww2");
      if (player.sprite != null)
      {
	player.sprite.shadowOn = true;
	player.sprite.glowColor = null;
	player.sprite.updateFilters();
      }
      player.sprite = sprite;
      player.location = to;
      if (fromWarp)
      {
	SoundAssets.playSound("bubble-clap", 0.5);
      }
    }
    public function showPlayerForward(player:Player, completion:Number) { }
    public function endPlayerForward(player:Player) { }
    public function beginPlayerReverse(player:Player) { }
    public function showPlayerReverse(player:Player, completion:Number) { }
    public function endPlayerReverse(player:Player)
    {
      player.sprite = from.contentsSprite;
      player.location = from;
      if (fromWarp)
      {
        trace("... now warping to time " + warpTime);
        player.isWarping = true;
        player.warpTime = warpTime;
	player.warpLocation = from;
	player.warpDepartureLocation = to;

	SoundAssets.beginPlayingSound("warp-wiggle", 0.25, 0, 65535, "ww1");
	SoundAssets.beginPlayingSound("warp-wiggle", 0.1, 10, 65535, "ww2");
      }
    }

    private function showWorld(completion:Number)
    {
      to.setShift(-dx*(1.0-completion), -dy*(1.0-completion));
      to.setBlur(int(16.0*(1.0-completion)));
    }

    public function testWorldForward(maze:Maze):Array
    {
      if (to.contents != MazeSquare.EMPTY)
      {
        return [from, to, "arrive"];
      }
      return null;
    }

    public function beginWorldForward(maze:Maze)
    {
      to.contentsSprite = sprite;
      to.contents = MazeSquare.OCCUPIED;
      stage.addChild(sprite);
      showWorld(0.0);
    }

    public function showWorldForward(maze:Maze, completion:Number) { showWorld(completion); }

    public function endWorldForward(maze:Maze)
    {
      to.contents = MazeSquare.PLAYER;
      showWorld(1.0);
    }

    public function beginWorldReverse(maze:Maze)
    {
      showWorld(1.0);
      to.contents = MazeSquare.OCCUPIED;
    }

    public function showWorldReverse(maze:Maze, completion:Number) { showWorld(completion); }

    public function endWorldReverse(maze:Maze)
    {
      trace("end world reverse " + this);
      stage.removeChild(sprite);
      showWorld(0.0);
      to.contentsSprite = null;
      to.contents = MazeSquare.EMPTY;
    }

    public function toString():String
    {
      return "warp arrival: " + from.xcoord + "," + from.ycoord + "-->" + to.xcoord + "," + to.ycoord;
    }
  }
}

