
package TimeTravel
{
  import TimeTravel.Player;
  import TimeTravel.MazeSquare;
  import flash.filters.*;
   
  public class PlayerMoveEvent implements PlayerEvent, WorldEvent
  {
    var from:MazeSquare, to:MazeSquare;    
    var linkedWorldEvents:Array = new Array();
    var dx:int, dy:int;

    public function PlayerMoveEvent(from:MazeSquare, to:MazeSquare)
    {
      this.from = from;
      this.to = to;
      dx = to.xcoord - from.xcoord;
      dy = to.ycoord - from.ycoord;
      linkWorldEvent(this);
    }

    public function linkWorldEvent(event:WorldEvent)
    {
      linkedWorldEvents.push(event);
    }

    public function getLinkedWorldEvents():Array
    {
      return linkedWorldEvents.slice();
    }

    public function beginPlayerForward(player:Player) { player.location = to; }
    public function showPlayerForward(player:Player, completion:Number) { }
    public function endPlayerForward(player:Player) { }
    public function beginPlayerReverse(player:Player) { }
    public function showPlayerReverse(player:Player, completion:Number) { }
    public function endPlayerReverse(player:Player) { player.location = from; }

    private function showWorld(completion:Number)
    {
      to.setShift(-dx*(1.0-completion), -dy*(1.0-completion));
      // to.setBlur(0);
    }

    public function testWorldForward(maze:Maze):Array
    {
      if ((to.contents != MazeSquare.EMPTY && to.contents != MazeSquare.DOOR) || from.contents != MazeSquare.PLAYER)
      {
         return [from, to, "move"];
      }
      return null;
    }

    public function beginWorldForward(maze:Maze)
    {
      to.contentsSprite = from.contentsSprite;
      to.contents = MazeSquare.OCCUPIED;
      from.contentsSprite = null;
      from.contents = MazeSquare.OCCUPIED;
      showWorld(0.0);
    }

    public function showWorldForward(maze:Maze, completion:Number) { showWorld(completion); }

    public function endWorldForward(maze:Maze)
    {
      to.contents = MazeSquare.PLAYER;
      from.contents = (from.doorSprite==null ? MazeSquare.EMPTY : MazeSquare.DOOR);
      if (from.contents == MazeSquare.DOOR && from.doorClosed)
      {
	from.doorSprite.fade = 1.0;
	from.doorSprite.updateFilters();
	SoundAssets.playSound("close-door", 0.5);
      }
      showWorld(1.0);
    }
    
    public function beginWorldReverse(maze:Maze)
    {
      showWorld(1.0);
      if (from.contents == MazeSquare.DOOR && from.doorClosed)
      {
	from.doorSprite.fade = 0.25;
	from.doorSprite.updateFilters();
      }
      from.contents = MazeSquare.OCCUPIED;
      to.contents = MazeSquare.OCCUPIED;
    }
    
    public function showWorldReverse(maze:Maze, completion:Number) { showWorld(completion); }

    public function endWorldReverse(maze:Maze)
    {
      showWorld(0.0);
      from.contentsSprite = to.contentsSprite;
      from.contents = MazeSquare.PLAYER;
      to.contentsSprite = null;
      to.contents = (to.doorSprite==null ? MazeSquare.EMPTY : MazeSquare.DOOR);
    }
  }
}

