
package TimeTravel
{
  import TimeTravel.Player;
  import TimeTravel.MazeSquare;
  import flash.filters.*;
  
  public class SlideObjectEvent implements PlayerEvent, WorldEvent
  {
    var playerLoc:MazeSquare, from:MazeSquare, to:MazeSquare;    
    var player:Player;
    var objectType:String;
    var linkedWorldEvents:Array = new Array();
    var dx:int, dy:int;

    public function SlideObjectEvent(from:MazeSquare, to:MazeSquare, player:Player, objectType:String)
    {
      this.from = from;
      this.to = to;
      this.playerLoc = player.location;
      this.player = player;
      this.objectType = objectType;
      dx = to.xcoord - from.xcoord;
      dy = to.ycoord - from.ycoord;
      linkWorldEvent(this);
    }

    public function linkWorldEvent(event:WorldEvent)
    {
      linkedWorldEvents.push(event);
    }

    public function getLinkedWorldEvents():Array
    {
      return linkedWorldEvents.slice();
    }

    public function beginPlayerForward(player:Player)
    {
      //SoundAssets.playSound("clearing-throat", 1.0);
      player.location = from;
    }
    public function showPlayerForward(player:Player, completion:Number) { }
    public function endPlayerForward(player:Player) { }
    public function beginPlayerReverse(player:Player) { }
    public function showPlayerReverse(player:Player, completion:Number) { }
    public function endPlayerReverse(player:Player)
    {
       player.location = playerLoc;
    }

    private function showWorld(completion:Number)
    {
      if (completion > 0.2)
      {
        to.setShift(-dx*(1.0-1.25*(completion-0.2)), -dy*(1.0-1.25*(completion-0.2)));
      }
      else to.setShift(-dx, -dy);
      from.setShift(-dx*(1.0-completion), -dy*(1.0-completion));
    }

    public function testWorldForward(maze:Maze):Array
    {
      if (playerLoc.contents != MazeSquare.PLAYER || from.contents != objectType)
      {
        return [playerLoc, from, "slide-object-1"];
      }
      if (to.contents != MazeSquare.EMPTY)
      {
        return [from, to, "slide-object-2"];
      }
      return null;
    }

    public function beginWorldForward(maze:Maze)
    {
      player.stage.removeChild(from.contentsSprite);
      player.stage.removeChild(playerLoc.contentsSprite);
      player.stage.addChild(from.contentsSprite);
      player.stage.addChild(playerLoc.contentsSprite);

      to.contentsSprite = from.contentsSprite;
      to.contents = MazeSquare.OCCUPIED;
      from.contentsSprite = playerLoc.contentsSprite;
      from.contents = MazeSquare.OCCUPIED;
      playerLoc.contentsSprite = null;
      playerLoc.contents = MazeSquare.OCCUPIED;
      showWorld(0.0);
    }

    public function showWorldForward(maze:Maze, completion:Number) { showWorld(completion); }

    public function endWorldForward(maze:Maze)
    {
      to.contents = objectType;
      from.contents = MazeSquare.PLAYER;
      to.blockIsLinked = from.blockIsLinked;
      from.blockIsLinked = [];
      playerLoc.contents = MazeSquare.EMPTY;
      showWorld(1.0);
    }

    public function beginWorldReverse(maze:Maze)
    {
      showWorld(1.0);
      playerLoc.contents = MazeSquare.OCCUPIED;
      from.blockIsLinked = to.blockIsLinked;
      to.blockIsLinked = [];
      from.contents = MazeSquare.OCCUPIED;
      to.contents = MazeSquare.OCCUPIED;
    }

    public function showWorldReverse(maze:Maze, completion:Number) { showWorld(completion); }

    public function endWorldReverse(maze:Maze)
    {
      showWorld(0.0);
      playerLoc.contentsSprite = from.contentsSprite;
      playerLoc.contents = MazeSquare.PLAYER;
      from.contentsSprite = to.contentsSprite;
      from.contents = objectType;
      to.contentsSprite = null;
      to.contents = MazeSquare.EMPTY;
    }
  }
}

