
package TimeTravel
{
  import flash.display.*;
  import flash.events.*;
  import flash.filters.*;
  import flash.ui.*;
  import flash.text.*;
  import UserInterface.*;
  
  public class TTApp extends Sprite implements Chapter
  {
    public static var RETURN_TO_MENU:String = "return-to-menu";
    public static var SUCCESSFUL_COMPLETION:String = "successful-completion";
    
    var maze:Maze;
    var mazeImage:FilteredSprite;
    var player:Player;
    var recordedSight:Array = [];
    
    var keyPressed:* = null;

    var theStage:Stage;
    var levelName:String;
    public var levelNumber:int;
    
    var visibleSquares:Array;

    private static var STOP_MODE:String = "stop";
    private static var FORWARD_MODE:String = "forward";
    private static var REVERSE_MODE:String = "reverse";
    private static var PARADOX_MODE:String = "paradox";
    private static var RESET_LEVEL_MODE:String = "reset-level";
    
    private var playbackMode:String = STOP_MODE;
    
    private function createInventoryField(x:int, y:int):TextField
    {
      var tf:TextField = new TextField();
      tf.x = x;
      tf.y = y;
      tf.text = "";
      var format:TextFormat = tf.getTextFormat();
      format.font = new Fonts().phoenixFont.fontName;
      format.align = TextFormatAlign.LEFT;
      format.size = 28;
      format.color = 0xFFFFFF;
      tf.autoSize = TextFieldAutoSize.LEFT;
      tf.setTextFormat(format);
      tf.defaultTextFormat = format;
      tf.embedFonts = true;
      addChild(tf);
      return tf;
    }

    private var ag:AnimatingGradient;
    
    private function tweenAgain(event:Event):void
    {
      this.ag.tweenGradient(ag.eColor, ag.sColor, 72 + 24*Math.random());
    }

    private var playerclock:TextField;

    public function TTApp(stage:Stage = null, levelInfo:LevelInfo = null, levelAvailable:Boolean = true)
    {
      if (stage != null) { theStage = stage; }
      else { theStage = this.stage; }

      theStage.scaleMode = StageScaleMode.NO_SCALE;
      theStage.align = StageAlign.TOP_LEFT;
      // this.ag.tweenGradient(0x000099 + Math.random()*0x000066, 0x000099 + Math.random()*0x000066, 48);

      if (levelAvailable) {
	this.ag = new AnimatingGradient(theStage.stageWidth, theStage.stageHeight, 0xFF0000, 0x0000FF);
	this.addChild(this.ag);
	this.ag.tweenGradient(0xFF0000, 0x0000FF, 72 + 24*Math.random());
      }
      else {
	this.ag = new AnimatingGradient(theStage.stageWidth, theStage.stageHeight, 0x000000, 0xFFFFFF);
	this.addChild(this.ag);
	this.ag.tweenGradient(0x000000, 0xFFFFFF, 72 + 24*Math.random());
      }
      
      this.ag.addEventListener("finish", tweenAgain);
      
      if (levelInfo == null)
      {
	maze = Maze.createRandomMaze(7, 5, 0.75);
	levelName = "Random";
	levelNumber = -1;
      }
      else
      {
	maze = Maze.createMazeFromLevelInfo(levelInfo);
	levelName = levelInfo.name;
	levelNumber = levelInfo.levelNumber;
      }
      
      mazeImage = new FilteredSprite();

      mazeImage.shadowOn = true;
      mazeImage.updateFilters();
      
      addChild(mazeImage);

      for each (var msq:MazeSquare in maze.getAllLocations())
      { 
	if (msq.starsNeeded > 0)
	{
	  msq.starsSprite = Sprites.createNamedSprite(msq.starsReceived + "-of-" + msq.starsNeeded + "-stars");
	  msq.starsSprite.shadowOn = false;
	  msq.starsSprite.updateFilters();
	  msq.update();
	  addChild(msq.starsSprite);
	}

	if (msq.contentsSprite != null)
	{
	  addChild(msq.contentsSprite);
        }
	
	msq.paint(mazeImage.graphics);
	msq.setParentShape(this);
	msq.setVisible(false);
      }

      var playerInventory:Object = new Object();
      var playerInventorySprite:Object = new Object();

      var inventory1:TextField = createInventoryField(stage.stageWidth - 50, 60);
      var inventory2:TextField = createInventoryField(stage.stageWidth - 50, 150);
      var inventory3:TextField = createInventoryField(stage.stageWidth - 50, 240);

      playerInventory["light-block"] = inventory1;
      playerInventorySprite["light-block"] = Sprites.createNamedSprite("light-block", 72);
      playerInventorySprite["light-block"].x = inventory1.x - 20;
      playerInventorySprite["light-block"].y = inventory1.y;
      removeChild(playerInventory["light-block"])
      addChild(playerInventorySprite["light-block"])
      addChild(playerInventory["light-block"])
      playerInventory["light-block"].filters = [new GlowFilter(0x000000, 1.0, 12, 12)];

      playerInventory["key"] = inventory2;
      playerInventorySprite["key"] = Sprites.createNamedSprite("key", 72);
      playerInventorySprite["key"].x = inventory2.x - 20;
      playerInventorySprite["key"].y = inventory2.y;
      removeChild(playerInventory["key"])
      addChild(playerInventorySprite["key"])
      addChild(playerInventory["key"])
      playerInventory["key"].filters = [new GlowFilter(0x000000, 1.0, 12, 12)];

      playerInventory["stars"] = inventory3;
      playerInventorySprite["stars"] = Sprites.createNamedSprite("stars", 72);
      playerInventorySprite["stars"].x = inventory3.x - 20;
      playerInventorySprite["stars"].y = inventory3.y;
      removeChild(playerInventory["stars"])
      addChild(playerInventorySprite["stars"])
      addChild(playerInventory["stars"])
      playerInventory["stars"].filters = [new GlowFilter(0x000000, 1.0, 12, 12)];

      player = new Player(maze, playerInventory, playerInventorySprite, this);

      playerclock = createInventoryField(40, stage.stageHeight - 75);

      var menuButton:EasyButton = new EasyButton("BACK", 14, 0x000000, 4);
      menuButton.x = stage.stageWidth - 35;
      menuButton.y = stage.stageHeight - 25;
      addChild(menuButton);
      menuButton.addEventListener(MouseEvent.CLICK, clickMenuButton);

      var instructionButton:EasyButton = new EasyButton("HELP", 14, 0x000000, 4);
      instructionButton.x = stage.stageWidth - 80;
      instructionButton.y = stage.stageHeight - 25;
      addChild(instructionButton);
      instructionButton.addEventListener(MouseEvent.CLICK, clickInstructionButton);

      var resetButton:EasyButton = new EasyButton("RESET", 14, 0x000000, 4);
      resetButton.x = stage.stageWidth - 130;
      resetButton.y = stage.stageHeight - 25;
      addChild(resetButton);
      resetButton.addEventListener(MouseEvent.CLICK, resetLevel);
    }

    function resetLevel(event:Event)
    {
      SoundAssets.playSound("release-click", 0.5);
      playbackMode = RESET_LEVEL_MODE;
    }
    
    function clickMenuButton(event:Event)
    {
      SoundAssets.playSound("r-plastic", 0.5);
      this.dispatchEvent(new Event(RETURN_TO_MENU));
      SoundAssets.stopPlayingSound("ww1");
      SoundAssets.stopPlayingSound("ww2");
    }
    
    var ins:Instructions = null;
    var insFade:Number = 0.0;
    
    function clickInstructionButton(event:Event)
    {
      if (ins != null) return;
      SoundAssets.playSound("release-click", 0.5);
      SoundAssets.stopPlayingSound("ww1");
      SoundAssets.stopPlayingSound("ww2");
      ins = new Instructions(theStage);
      insFade = 0.0;
      ins.fade = insFade;
      ins.updateFilters();
      addChild(ins);
      ins.addEventListener(TTApp.RETURN_TO_MENU, closeInstructions);
    }

    function closeInstructions(event:Event)
    {
      insFade = -0.01;
    }
    
    var chapterParent:Sprite;
    
    public function addToParent(sprite:Sprite):void
    {
      chapterParent = sprite;
      chapterParent.addChild(this);
    }

    public function getParent():Sprite
    {
      return chapterParent;
    }

    public function getDisplay():Sprite
    {
      return this;
    }

    private var yval:int;
    
    public function makeActive():void
    {
      theStage.addEventListener(KeyboardEvent.KEY_DOWN, keyDown);
      theStage.addEventListener(KeyboardEvent.KEY_UP, keyUp);
      theStage.focus = theStage;
      
      yval = theStage.stageHeight;
      addEventListener(Event.ENTER_FRAME, tick);

      if (levelNumber == 1)
      {
	setHelp("Head for the exit!  (Use arrow keys to move.)");
      }
      else if (levelNumber == 2)
      {
	setHelp("Doors will open and close as you wait.");
      }
      else if (levelNumber == 3)
      {
	setHelp("The exit seems blocked...");
      }
      else if (levelNumber == 4)
      {
	setHelp("The linked blocks will slide together.");
      }
      else if (levelNumber == 5)
      {
	setHelp("The door takes two keys, but only one is here...");
      }
      else if (levelNumber == 6)
      {
	setHelp("The doors will open, in good time...");
      }
    }

    public function makeInactive():void
    {
      removeEventListener(Event.ENTER_FRAME, tick);

      theStage.removeEventListener(KeyboardEvent.KEY_DOWN, keyDown);
      theStage.removeEventListener(KeyboardEvent.KEY_UP, keyUp);
    }

    public function removeFromParent():void
    {
      chapterParent.removeChild(this);
      chapterParent = null;
    }

    private function checkHintLife()
    {
      if (hintPopup != null && hintPopupKeys == 0)
      {
	if (helpArray.length == 0 || helpArrayIndex >= helpArray.length)
	{
	  clearHelp();
	}
      }
    }

    function keyDown(event:KeyboardEvent)
    {
      if (keyPressed == null) keyPressed = event.keyCode;
      trace("key = " + event.keyCode);
      if (hintPopupKeys > 0)
      {
	--hintPopupKeys;
      }
      checkHintLife();
    }

    function keyUp(event:KeyboardEvent)
    {
      if (keyPressed == event.keyCode) keyPressed = null;
    }

    private function stepForward(amount:int)
    {
      if (player.isWarping && player.warpLocation.contents != MazeSquare.EXIT_SQUARE && !(player.getNextEvent() != null && player.getNextEvent() instanceof PlayerArrivesEvent))
      {
	maze.rewindWorldTime(amount);
      }
      else
      {
	var ok:Boolean = maze.advanceWorldTime(amount);
	if (!ok) {
	  playbackMode = PARADOX_MODE;
	  player.removeEventsFromPlayerTimeline();
	  return;
	}
      }

      player.advancePlayerTime(amount);
    }
    
    private function stepReverse(amount:int)
    {
      if (player.isWarping && player.warpTime != maze.currentTime - 1 && player.warpLocation.contents != MazeSquare.EXIT_SQUARE)
      {
	var ok:Boolean = maze.advanceWorldTime(amount);
	if (!ok) {
	  playbackMode = PARADOX_MODE;
	  player.removeEventsFromPlayerTimeline();
	  return;
	}
      }
      else
      {
	maze.rewindWorldTime(amount);
      }

      player.rewindPlayerTime(amount);
    }

    public var hintPopup:BubbleStrip = null;
    public var hintPopupKeys:int = 0;
    public var hintPopupTime:int = 0;
    public var helpArray:Array = [];
    public var helpArrayIndex:int = 0;

    private function setHelpIndex(ix:int)
    {
      helpArrayIndex = ix;
      setHelp(helpArray[helpArrayIndex % helpArray.length], false);
      checkHintLife();
    }
    
    private function setMultiHelp(helps:Array)
    {
      helpArray = helps.slice();
      helpArray.push("...");
      setHelp(helpArray[0]);
      setHelpIndex(0);
      hintPopupKeys = 3;
    }
    
    private function setHelp(help:String, sound:Boolean = true)
    {
      if (hintPopup != null) removeChild(hintPopup);
      hintPopup = new BubbleStrip(help, this);
      addChild(hintPopup);
      // if (sound) SoundAssets.playSound("bubble-clap", 0.5);
      theStage.focus = theStage;
      if (sound) hintPopupKeys = 3;
    }

    private function clearHelp()
    {
      if (hintPopup != null) removeChild(hintPopup);
      hintPopup = null;
      helpArray = [];
      helpArrayIndex = 0;
      hintPopupKeys = 0;
      hintPopupTime = 0;
    }

    public function tick(event:Event)
    {
      if (levelNumber == 1 && maze.currentTime >= 5 && !LevelManager.closedDoorHintViewed)
      {
	setMultiHelp(["The door closed by itself!...  (more)", "You need to reach it earlier in time."]);
	LevelManager.closedDoorHintViewed = true;
      }

      if (playbackMode == PARADOX_MODE && maze.currentParadox[2]=="eye" && !LevelManager.eyeParadoxHintViewed)
      {
	setMultiHelp(["A visual paradox!...  (more)", "A previous self sees something he didn't before.  (more)", "[SHIFT] to undo and try something else."]);
	LevelManager.eyeParadoxHintViewed = true;
      }

      if (playbackMode == PARADOX_MODE && maze.currentParadox[2]!="eye" && !LevelManager.moveParadoxHintViewed)
      {
	setMultiHelp(["A physical paradox!...  (more)", "Something can't move the way it did before.  (more)", "[SHIFT] to undo and try something else."]);
	LevelManager.moveParadoxHintViewed = true;
      }

      if (player.inventory[MazeSquare.LIGHT_BLOCK]>0 && !LevelManager.lightBlockHintViewed)
      {
	setMultiHelp(["You are carrying a light block.  (more)", "Press [1] and a direction to place it.  (more)", "Remember that you can't see through it..."]);
	LevelManager.lightBlockHintViewed = true;
      }
      
      if (ins != null)
      {
	if (insFade >= 0.0 && insFade < 1.0)
	{
	  insFade += 1.0/12.0;
	  ins.fade = insFade;
	  ins.updateFilters();
	}
	else if (insFade < 0.0)
	{
	  insFade -= 1.0/12.0;
	  ins.fade = 1.0 + insFade;
	  ins.updateFilters();
	  if (insFade < -1.0)
	  {
	    removeChild(ins);
	    ins = null;
	    insFade = 0.0;
	    theStage.focus = theStage;
	  }
	}
	return;
      }
      
      if (hintPopup != null && keyPressed == Keyboard.ENTER)
      {
	clearHelp();
      }

      if (hintPopup != null && helpArray.length > 0)
      {
	hintPopupTime = (hintPopupTime + 1)%(24*3);
	if (hintPopupTime == 0)
	{
	  setHelpIndex(helpArrayIndex+1);
	}
      }

      if (player.isWarping && !LevelManager.warpHintViewed)
      {
	setMultiHelp(["YOU'RE in a temporal rift!  (more)", "[LEFT] to travel back in time...  (more)", "[RIGHT] to go forward in time...  (more)", "[SPACE] to leave the warp."]);
	SoundAssets.stopPlayingSound("ww1");
	SoundAssets.stopPlayingSound("ww2");
	LevelManager.warpHintViewed = true;
	return;
      }
      
      if ((player.location != null && player.location.starsNeeded > 0 && !LevelManager.starsHintViewed) ||
	  (player.inventory[MazeSquare.STARS]>0 && !LevelManager.starsHintViewed))
      {
	setMultiHelp(["Placing stars in empty slots can clear the way...  (more)", "Press [3] and a direction to place stars."]);
	LevelManager.starsHintViewed = true;
      }

      if (player.inventory[MazeSquare.KEY]>0 && !LevelManager.keyHintViewed)
      {
	setMultiHelp(["You got a key!...  (more)", "Keys can open locked doors.  (more)", "Press [2] and a direction to drop a key."]);
	LevelManager.keyHintViewed = true;
      }
      
      if (player.sprite != null)
      {
      	player.sprite.shadowOn = true;
	player.sprite.glowColor = null;

	if (playbackMode == REVERSE_MODE || playbackMode == RESET_LEVEL_MODE) player.sprite.glowColor = "blue";
	else if (player.isWarping || player.isDropping != MazeSquare.EMPTY) player.sprite.glowColor = "red";
	else if (playbackMode == FORWARD_MODE) player.sprite.glowColor = "green";
	
	player.sprite.updateFilters();
      }

      if (player.floatingSprite != null)
      {
	player.floatingSprite.shadowOn = true;
	player.floatingSprite.fade = 0.5;
	player.floatingSprite.rotation = (player.floatingSprite.rotation + 10) % 360;
	player.floatingSprite.updateFilters();
      }
      
      if (player.getNextEvent() != null && playbackMode == STOP_MODE)
      {
	playbackMode = FORWARD_MODE;
      }
      
      if (playbackMode == FORWARD_MODE)
      {
	stepForward(6);
	if (player.currentTimeExtra == 0 && playbackMode != PARADOX_MODE)
	{
	  playbackMode = STOP_MODE;
	}
      }

      if (playbackMode == REVERSE_MODE)
      {
	stepReverse(12);
	if (player.currentTimeExtra == 0 && playbackMode != PARADOX_MODE)
	{
	  playbackMode = STOP_MODE;
	}
      }

      if (playbackMode == RESET_LEVEL_MODE)
      {
	stepReverse(24);
	if (player.currentTimeExtra == 0 && player.currentTime < 1)
	{
	  playbackMode = STOP_MODE;
	}
      }

      if (player.currentTimeExtra == 0 && playbackMode == STOP_MODE)
      {
	if (player.isWarping)
	{
	  visibleSquares = [player.location];
	}
	else
	{
	  visibleSquares = maze.getVisibleSquares(player.location);
	}
	
	for each (var msq:MazeSquare in maze.getAllLocations())
	{
	  msq.setVisible(true);
	  msq.setVisible(false);
	}

	for each (msq in visibleSquares)
	{
	  msq.setVisible(true);
	}
	
	for each (msq in maze.getAllLocations())
	{
	  if (msq.contents == MazeSquare.PLAYER)
	  {
	    var vs:Array = maze.getVisibleSquares(msq);
	    var minDist:Number = -1.0;
	    for each (var msq2:MazeSquare in vs)
	    {
	      if (maze.recordedSight[maze.currentTime] && maze.recordedSight[maze.currentTime][msq2.x] && maze.recordedSight[maze.currentTime][msq2.x][msq2.y] && msq2.contents != maze.recordedSight[maze.currentTime][msq2.x][msq2.y][1])
	      {
		playbackMode = PARADOX_MODE;
		var origin:MazeSquare = maze.recordedSight[maze.currentTime][msq2.x][msq2.y][0];
	        var dx:int = origin.x - msq2.x;
		var dy:int = origin.y - msq2.y;
		var dist:Number = Math.sqrt(1.0*dx*dx + 1.0*dy*dy);
		if (minDist < 0.0 || dist < minDist)
		{
		  minDist = dist;
		  maze.setParadox([maze.recordedSight[maze.currentTime][msq2.x][msq2.y][0], msq2, "eye", maze.recordedSight[maze.currentTime][msq2.x][msq2.y][1]]);
		}
	      }
	    }
	  }
	}

	if (playbackMode != PARADOX_MODE && !player.isWarping)
	{
	  var newObs:Array = [];
	  for each (msq in visibleSquares)
	  {
	    if (!maze.recordedSight[maze.currentTime] || !maze.recordedSight[maze.currentTime][msq.x] || !maze.recordedSight[maze.currentTime][msq.x][msq.y])
	    {
	      newObs.push([maze.currentTime, player.location, msq.x, msq.y, msq.contents]);
	    }
	  }
	  if (newObs.length > 0 && (player.currentTime == 1 || player.getPreviousEvent() != null))
	  {
	    var newObsEvent:ObservationEvent = new ObservationEvent(newObs, maze);
	    newObsEvent.beginPlayerForward(player);
	    player.addEventToTime(newObsEvent, player.currentTime - 1);
	  }
	}
      }

      if (playbackMode == PARADOX_MODE)
      {
	addChild(maze.paradoxSprite);
      }
      else if (maze.paradoxSprite != null)
      {
	removeChild(maze.paradoxSprite);
	maze.paradoxSprite = null;
      }
      
      if (playbackMode == STOP_MODE && player.currentTimeExtra == 0)
      {
	if (keyPressed == Keyboard.SHIFT && player.getPreviousEvent() != null)
	{
	  playbackMode = REVERSE_MODE;
	}
	else if (!player.isWarping)
	{
	  if (keyPressed == Keyboard.UP) player.move(0, -1);
	  else if (keyPressed == Keyboard.DOWN) player.move(0, 1);
	  else if (keyPressed == Keyboard.LEFT) player.move(-1, 0);
	  else if (keyPressed == Keyboard.RIGHT) player.move(1, 0);
	  else if (keyPressed == Keyboard.SPACE) player.move(0, 0);
	  else if (keyPressed == 49 && player.inventory[MazeSquare.LIGHT_BLOCK]>0) player.isDropping = MazeSquare.LIGHT_BLOCK;
	  else if (keyPressed == 50 && player.inventory[MazeSquare.KEY]>0) player.isDropping = MazeSquare.KEY;
	  else if (keyPressed == 51 && player.inventory[MazeSquare.STARS]>0) player.isDropping = MazeSquare.STARS;
	}
	else
	{
	  // player is warping
	  if (keyPressed == Keyboard.LEFT) player.move(0, 0);
	  else if (keyPressed == Keyboard.RIGHT && maze.currentTime <= player.warpTime)
	  {
	    playbackMode = REVERSE_MODE;
	  }
	  else if (keyPressed == Keyboard.SPACE)
	  {
	    player.dropOutOfWarp();
	  }
	}
      }
      
      if (playbackMode == PARADOX_MODE && player.currentTimeExtra == 0 && keyPressed == Keyboard.SHIFT)
      {
	playbackMode = REVERSE_MODE;
      }
      
      if (!player.isWarping && player.location.contentsSprite == null && playbackMode == STOP_MODE && keyPressed != Keyboard.SHIFT)
      {
	playbackMode = FORWARD_MODE;
      }
      
      var playerLeftLevel:Boolean = (player.isWarping && player.warpLocation != null && player.warpLocation.contents == MazeSquare.EXIT_SQUARE);
      
      if (playerLeftLevel && playbackMode == STOP_MODE)
      {
	playbackMode = (keyPressed==Keyboard.SHIFT ? REVERSE_MODE : FORWARD_MODE);
      }
      
      mazeImage.shadowOn = true;
      mazeImage.blurX = mazeImage.blurY = 0;
      mazeImage.glowColor = null;
      if (player.isWarping)
      {
	mazeImage.blurX = mazeImage.blurY = 4;
	mazeImage.glowColor = "red";
      }
      mazeImage.updateFilters();
      
      // playerclock.text = levelName + "\nTime: " + maze.currentTime + ":" + maze.currentTimeExtra + " / " + player.currentTime + ":" + player.currentTimeExtra + "  " + (playbackMode!=PARADOX_MODE ? playbackMode : maze.currentParadox[2]) + "  " + (player.getNextEvent() != null ? "T" : "F");
      
      playerclock.text = levelName;
      if (playbackMode == RESET_LEVEL_MODE) playerclock.text += "\n... RESETTING LEVEL ...";
      else playerclock.text += "\nTime: " + maze.currentTime;

      player.update();

      if (playerLeftLevel && maze.currentTime > maze.getLastWorldEventTime() + 4)
      {
	this.dispatchEvent(new Event(SUCCESSFUL_COMPLETION));
	this.dispatchEvent(new Event(RETURN_TO_MENU));
	SoundAssets.stopPlayingSound("ww1");
	SoundAssets.stopPlayingSound("ww2");
      }
    }
  }
}

