
package TimeTravel
{
  import TimeTravel.Player;
  import TimeTravel.MazeSquare;
  import flash.display.*;
  
  public class TakeObjectEvent implements WorldEvent, PlayerEvent
  {
    var at:MazeSquare;
    var playerLoc:MazeSquare;
    var objectType:String;
    var player:Player;
    var linkedWorldEvents:Array = new Array();
    var sprite:FilteredSprite;
    var dx:int, dy:int;

    public function TakeObjectEvent(at:MazeSquare, player:Player, objectType:String)
    {
      this.at = at;
      this.objectType = objectType;
      this.player = player;
      this.playerLoc = player.location;
      dx = playerLoc.xcoord - at.xcoord;
      dy = playerLoc.ycoord - at.ycoord;
      linkWorldEvent(this);

      sprite = at.contentsSprite;
    }

    public function linkWorldEvent(event:WorldEvent)
    {
      linkedWorldEvents.push(event);
    }

    public function getLinkedWorldEvents():Array
    {
      return linkedWorldEvents.slice();
    }

    public function beginPlayerForward(player:Player)
    {
      player.inventory[objectType] += 1;
      SoundAssets.playSound(objectType==MazeSquare.STARS ? "glass-click" : "metal-hit", 0.25);
    }
    
    public function showPlayerForward(player:Player, completion:Number) { }
    public function endPlayerForward(player:Player) { }
    public function beginPlayerReverse(player:Player) { }
    public function showPlayerReverse(player:Player, completion:Number) { }
    public function endPlayerReverse(player:Player)
    {
      player.inventory[objectType] -= 1;
      // SoundAssets.playSound(objectType==MazeSquare.STARS ? "glass-click" : "metal-hit", 0.5);
    }

    private function showWorld(completion:Number)
    {
       at.setShift(0.5*dx*completion, 0.5*dy*completion);
       at.setBlur(int(16.0*completion));
    }

    public function testWorldForward(maze:Maze):Array
    {
       if (at.contents != objectType || playerLoc.contents != MazeSquare.PLAYER)
       {
          return [playerLoc, at, "take"];
       }
       return null;
    }

    public function beginWorldForward(maze:Maze)
    {
      at.contents = MazeSquare.OCCUPIED;
      showWorld(0.0);
    }

    public function showWorldForward(maze:Maze, completion:Number) { showWorld(completion); }

    public function endWorldForward(maze:Maze)
    {
      showWorld(1.0);
      at.contents = MazeSquare.EMPTY;
      sprite = at.contentsSprite;
      at.contentsSprite = null;
      player.stage.removeChild(sprite);
    }
    
    public function beginWorldReverse(maze:Maze)
    {
      player.stage.addChild(sprite);
      at.contentsSprite = sprite;
      at.contents = MazeSquare.OCCUPIED;
      showWorld(1.0);
    }

    public function showWorldReverse(maze:Maze, completion:Number) { showWorld(completion); }

    public function endWorldReverse(maze:Maze)
    {
      showWorld(0.0);
      at.contents = objectType;
    }
  }
}

