
package TimeTravel
{
  import flash.display.*;
  import flash.events.*;
  import flash.filters.*;
  import flash.ui.*;
  import flash.text.*;
  import UserInterface.*;
  
  public class TTApp extends Sprite implements Chapter
  {
    public static var RETURN_TO_MENU:String = "return-to-menu";
    public static var SUCCESSFUL_COMPLETION:String = "successful-completion";
    
    var maze:Maze;
    var mazeImage:FilteredSprite;
    var player:Player;

    var isRewinding:Boolean;
    var keyPressed:* = null;

    var theStage:Stage;
    var levelName:String;
    public var levelNumber:int;
    
    var visibleSquares:Array;

    private static var STOP_MODE:String = "stop";
    private static var FORWARD_MODE:String = "forward";
    private static var REVERSE_MODE:String = "reverse";
    private static var PARADOX_MODE:String = "paradox";
    
    private var playbackMode:String = STOP_MODE;
    
    private function createInventoryField(x:int, y:int):TextField
    {
      var tf:TextField = new TextField();
      tf.x = x;
      tf.y = y;
      tf.text = "";
      var format:TextFormat = tf.getTextFormat();
      format.font = new Fonts().phoenixFont.fontName;
      format.align = TextFormatAlign.LEFT;
      format.size = 28;
      format.color = 0xFFFFFF;
      tf.autoSize = TextFieldAutoSize.LEFT;
      tf.setTextFormat(format);
      tf.defaultTextFormat = format;
      tf.embedFonts = true;
      addChild(tf);
      return tf;
    }

    private var ag:AnimatingGradient;
			
    private function tweenAgain(event:Event):void
    {
      this.ag.tweenGradient(ag.eColor, Math.random()*0xFFFFFF, 48);
    }

    private var playerclock:TextField;

    public function TTApp(stage:Stage = null, levelInfo:LevelInfo = null)
    {
      if (stage != null) { theStage = stage; }
      else { theStage = this.stage; }

      theStage.scaleMode = StageScaleMode.NO_SCALE;
      theStage.align = StageAlign.TOP_LEFT;
      this.ag = new AnimatingGradient(theStage.stageWidth, theStage.stageHeight, Math.random()*0xffffff, Math.random()*0xffffff);
      this.addChild(this.ag);
      this.ag.tweenGradient(0x000099 + Math.random()*0x000066, 0x000099 + Math.random()*0x000066, 48);
      this.ag.addEventListener("finish", tweenAgain);
      
      if (levelInfo == null)
      {
	maze = Maze.createRandomMaze(7, 5, 0.75);
	levelName = "Random";
	levelNumber = -1;
      }
      else
      {
	maze = Maze.createMazeFromLevelInfo(levelInfo);
	levelName = levelInfo.name;
	levelNumber = levelInfo.levelNumber;
      }
      
      mazeImage = new FilteredSprite();

      mazeImage.shadowOn = true;
      mazeImage.updateFilters();
      
      addChild(mazeImage);

      for each (var msq:MazeSquare in maze.getAllLocations())
      { 
	if (msq.contentsSprite != null)
	{
	  addChild(msq.contentsSprite);
        }
	
	if (msq.starsNeeded > 0)
	{
	  msq.starsSprite = Sprites.createNamedSprite(msq.starsReceived + "-of-" + msq.starsNeeded + "-stars");
	  msq.starsSprite.shadowOn = false;
	  msq.starsSprite.updateFilters();
	  msq.update();
	  addChild(msq.starsSprite);
	}
	
	msq.paint(mazeImage.graphics);
	msq.setParentShape(this);
	msq.setVisible(false);
      }

      var playerInventory:Object = new Object();
      var playerInventorySprite:Object = new Object();

      var inventory1:TextField = createInventoryField(stage.stageWidth - 50, 60);
      var inventory2:TextField = createInventoryField(stage.stageWidth - 50, 150);
      var inventory3:TextField = createInventoryField(stage.stageWidth - 50, 240);

      playerInventory["light-block"] = inventory1;
      playerInventorySprite["light-block"] = Sprites.createNamedSprite("light-block", 72);
      playerInventorySprite["light-block"].x = inventory1.x - 20;
      playerInventorySprite["light-block"].y = inventory1.y;
      removeChild(playerInventory["light-block"])
      addChild(playerInventorySprite["light-block"])
      addChild(playerInventory["light-block"])
      playerInventory["light-block"].filters = [new GlowFilter(0x000000, 1.0, 12, 12)];

      playerInventory["key"] = inventory2;
      playerInventorySprite["key"] = Sprites.createNamedSprite("key", 72);
      playerInventorySprite["key"].x = inventory2.x - 20;
      playerInventorySprite["key"].y = inventory2.y;
      removeChild(playerInventory["key"])
      addChild(playerInventorySprite["key"])
      addChild(playerInventory["key"])
      playerInventory["key"].filters = [new GlowFilter(0x000000, 1.0, 12, 12)];

      playerInventory["stars"] = inventory3;
      playerInventorySprite["stars"] = Sprites.createNamedSprite("stars", 72);
      playerInventorySprite["stars"].x = inventory3.x - 20;
      playerInventorySprite["stars"].y = inventory3.y;
      removeChild(playerInventory["stars"])
      addChild(playerInventorySprite["stars"])
      addChild(playerInventory["stars"])
      playerInventory["stars"].filters = [new GlowFilter(0x000000, 1.0, 12, 12)];

      player = new Player(maze, playerInventory, playerInventorySprite, this);

      playerclock = createInventoryField(40, stage.stageHeight - 75);

      var menuButton:EasyButton = new EasyButton("back", 12, 0xFF0000, 4);
      menuButton.x = stage.stageWidth - 30;
      menuButton.y = stage.stageHeight - 25;
      addChild(menuButton);
      menuButton.addEventListener(MouseEvent.CLICK, clickMenuButton);
    }

    function clickMenuButton(event:Event)
    {
      this.dispatchEvent(new Event(RETURN_TO_MENU));
    }
    
    var chapterParent:Sprite;
    
    public function addToParent(sprite:Sprite):void
    {
      chapterParent = sprite;
      chapterParent.addChild(this);
    }

    public function getParent():Sprite
    {
      return chapterParent;
    }

    public function getDisplay():Sprite
    {
      return this;
    }

    private var yval:int;
    
    public function makeActive():void
    {
      theStage.addEventListener(KeyboardEvent.KEY_DOWN, keyDown);
      theStage.addEventListener(KeyboardEvent.KEY_UP, keyUp);
      theStage.focus = theStage;
      
      yval = theStage.stageHeight;
      addEventListener(Event.ENTER_FRAME, tick);
    }

    public function makeInactive():void
    {
      removeEventListener(Event.ENTER_FRAME, tick);

      theStage.removeEventListener(KeyboardEvent.KEY_DOWN, keyDown);
      theStage.removeEventListener(KeyboardEvent.KEY_UP, keyUp);
    }

    public function removeFromParent():void
    {
      chapterParent.removeChild(this);
      chapterParent = null;
    }
 
    function keyDown(event:KeyboardEvent)
    {
      if (keyPressed == null) keyPressed = event.keyCode;
      trace("key = " + event.keyCode);
    }

    function keyUp(event:KeyboardEvent)
    {
      if (keyPressed == event.keyCode) keyPressed = null;
    }

    private function stepForward(amount:int)
    {
      if (player.isWarping)
      {
	maze.rewindWorldTime(amount);
      }
      else
      {
	maze.advanceWorldTime(amount);
      }

      player.advancePlayerTime(amount);
    }
    
    private function stepReverse(amount:int)
    {
      if (player.isWarping)
      {
	maze.advanceWorldTime(amount);
      }
      else
      {
	maze.rewindWorldTime(amount);
      }

      player.rewindPlayerTime(amount);
    }

    public var bubble:BubbleDialog = null;
    
    public function tick(event:Event)
    {
      if (yval > theStage.stageHeight / 2)
      {
	yval -= 10;

	if (bubble == null) 
	  {
	    bubble = new BubbleDialog(
				    "Doc Franklin!\n" +
				    "We need your help in the Time Lab!\n" +
				    "Come quickly!\n", this);

	    //tf2.width = 0.75 * stage.stageWidth;
	    //tf2.setCoords(stage.stageWidth / 2, 0);
	    //addChild(tf2);
	    
	    //bubble = new FilteredSprite();
	    //bubble.graphics.beginFill(0x000000);
	    //bubble.graphics.drawRoundRect(tf2.x - 12, tf2.y - 12, tf2.width + 24, tf2.height + 24, 0.05*tf2.width);
	    //bubble.graphics.endFill();
	    //bubble.graphics.beginFill(0xFFFFFF);
	    //bubble.graphics.drawRoundRect(tf2.x - 10, tf2.y - 10, tf2.width + 20, tf2.height + 20, 0.05*tf2.width);
	    //bubble.graphics.endFill();
	    //bubble.shadowOn = true;
	    //bubble.updateFilters();
	    //var hooded:FilteredSprite = Sprites.createNamedSprite("hooded");
	    //hooded.x = tf2.width / 2;
	    //hooded.y = tf2.y - 20;
	    //bubble.addChild(hooded);
	    //var doc:FilteredSprite = Sprites.createNamedSprite(MazeSquare.PLAYER);
	    //doc.x = tf2.width / 2 + 200;
	    //doc.y = tf2.y + tf2.height + 10;
	    //bubble.addChild(doc);
	    //bubble.addChild(tf2);
	    addChild(bubble);
	  }
	
	bubble.y = yval;
	bubble.blurY = 10;
	bubble.updateFilters();
	return;
      }
      else if (yval > 0) {
	yval -= 2;
	// bubble.y = yval;
	bubble.blurY = 0;
	bubble.updateFilters();
	return;
      }
      else if (yval > -48) {
	bubble.fade = (48.0 + yval)/48.0;
	bubble.updateFilters();
	yval -= 1;
	if (yval <= -48)
	  {
	    removeChild(bubble);
	  }
	return;
      }
      
      
      if (player.sprite != null)
      {
	player.sprite.shadowOn = true;

	if (isRewinding) player.sprite.glowColor = "blue";
	else if (player.isWarping || player.isDropping != MazeSquare.EMPTY) player.sprite.glowColor = "red";
	else if (keyPressed != null) player.sprite.glowColor = "green";
	else player.sprite.glowColor = null;

	player.sprite.updateFilters();
      }
      
      if (isRewinding)
      {
	if (keyPressed != Keyboard.SHIFT) isRewinding = false;
      }
      else if (player.isWarping && keyPressed == Keyboard.SHIFT) isRewinding = true;
      else if (!player.isWarping && player.currentTimeExtra == 0)
      {
	if (keyPressed == Keyboard.DOWN) player.move(0, 1);
	else if (keyPressed == Keyboard.UP) player.move(0, -1);
	else if (keyPressed == Keyboard.RIGHT) player.move(1, 0);
	else if (keyPressed == Keyboard.LEFT) player.move(-1, 0);
	else if (keyPressed == Keyboard.SPACE) player.move(0, 0);
	else if (keyPressed == Keyboard.SHIFT) isRewinding = true;
	else if (keyPressed == 49 && player.inventory[MazeSquare.LIGHT_BLOCK]>0) player.isDropping = MazeSquare.LIGHT_BLOCK;
	else if (keyPressed == 50 && player.inventory[MazeSquare.KEY]>0) player.isDropping = MazeSquare.KEY;
	else if (keyPressed == 51 && player.inventory[MazeSquare.STARS]>0) player.isDropping = MazeSquare.STARS;
      }

      var playerLeftLevel:Boolean = (player.warpLocation != null && player.warpLocation.contents == MazeSquare.EXIT_SQUARE);
      
      if (player.isWarping && !playerLeftLevel)
      { 
	trace("player is warping");
	if (isRewinding)
	{
	  if (maze.currentTime < player.warpTime)
	  {
	    maze.advanceWorldTime(17);
	  }
	  else
	  {
	    maze.rewindWorldTime(17);
	    player.rewindPlayerTime(17);
	  }
	}
	else if (keyPressed == Keyboard.LEFT) maze.rewindWorldTime(6);
	else if (keyPressed == Keyboard.RIGHT && maze.currentTime <= player.warpTime)
	{
	  maze.advanceWorldTime(6);
	}
	else if (keyPressed == Keyboard.SPACE) player.dropOutOfWarp();
      }
      else if (isRewinding)
        {
	  if (player.currentTime >= -1)
	    {
	      if (keyPressed == Keyboard.SHIFT || player.currentTimeExtra > 17) {
		maze.rewindWorldTime(17);
		player.rewindPlayerTime(17);
	      }
	      else {
		maze.rewindWorldTime(player.currentTimeExtra);
		player.rewindPlayerTime(player.currentTimeExtra);
		isRewinding = false;
	      }
	    }
        }
      else if (player.hasBufferedEvents())
        {
	  trace("buffered events at time " + player.currentTime + " " + player.currentTimeExtra);
	  player.applyBufferedEvents();
	  if (maze.advanceWorldTime(7))
	    player.advancePlayerTime(7);
	  else { maze.rewindWorldTime(24); player.rewindPlayerTime(24); }
        }
      else if (player.location.contentsSprite == null)
        {
	  if (maze.advanceWorldTime(7))
	    player.advancePlayerTime(7);
	  else { maze.rewindWorldTime(24); player.rewindPlayerTime(24); }
	  trace("no player... advancing to " + player.currentTimeExtra);
	  if (player.currentTimeExtra==0) { maze.advanceWorldTime(7); player.advancePlayerTime(7); }
        }
      else if (player.currentTimeExtra != 0)
        {
	  if (maze.advanceWorldTime(7))
	    player.advancePlayerTime(7);
	  else { maze.rewindWorldTime(24); player.rewindPlayerTime(24); }
        }
      
      mazeImage.shadowOn = true;
      mazeImage.blurX = mazeImage.blurY = 0;
      mazeImage.glowColor = null;
      if (player.isWarping || isRewinding) mazeImage.blurX = mazeImage.blurY = 4;
      if (player.isWarping) mazeImage.glowColor = "red";
      mazeImage.updateFilters();
      
      playerclock.text = levelName + "\nTime: " + maze.currentTime;
      
      player.update();
      
      // if (player.currentTimeExtra == 0)
      // {
	visibleSquares = maze.getVisibleSquares(player.location);
	for each (var msq:MazeSquare in maze.getAllLocations())
	{
	  msq.setVisible(false);
	}
	for each (msq in visibleSquares)
	{
	  msq.setVisible(true);
	}
	// }

	if (playerLeftLevel && maze.currentTime > maze.getLastWorldEventTime() + 4)
	{
	  this.dispatchEvent(new Event(SUCCESSFUL_COMPLETION));
	  this.dispatchEvent(new Event(RETURN_TO_MENU));
	}
    }
  }
}

