
package
{
  import flash.display.Sprite;
  import flash.text.*;
  import flash.events.*;
  import flash.media.SoundChannel;
  import flash.media.SoundTransform;
  import TimeTravel.*;
  import UserInterface.*;

  import com.kongregate.as3.client.KongregateAPI;
  import com.kongregate.as3.client.events.KongregateEvent;

  [SWF(width="550", height="400", backgroundColor="#FFFFFF")]

  public class TimeTravelMain extends Sprite
  {
    var kong:KongregateAPI = new KongregateAPI();

    var mainMenu:TimeTravelMainMenu;
    var currentLevel:TTApp;
    var prevChapter:Chapter;
    var nextChapter:Chapter;

    var levelSelect:LevelSelector = null;
    var lm:LevelManager = LevelManager.getInstance();

    var instructions:Instructions;
    var backstory:BackstoryChapter;
    
    var st:SoundTransform = new SoundTransform(1.0, 0.0);
    private var menuMusicChannel:SoundChannel;
    private var levelMusicChannel:SoundChannel;
    private var firstLevel:int = 1;
    
    private function playMusic(name:String)
    {
      if (name == "menu")
      {
	st.volume = 0.5;
	menuMusicChannel.soundTransform = st;
	st.volume = 0.0;
	levelMusicChannel.soundTransform = st;
      }
      else if (name == "level")
      {
	st.volume = 0.0;
	menuMusicChannel.soundTransform = st;
	st.volume = 1.0;
	levelMusicChannel.soundTransform = st;
      }
      else if (name == null)
      {
	st.volume = 0.0;
	menuMusicChannel.soundTransform = st;
	levelMusicChannel.soundTransform = st;
      }
    }
    
    public function TimeTravelMain()
    {
      menuMusicChannel = SoundAssets.createNamedAsset("menu-song").play(0, 65535);
      levelMusicChannel = SoundAssets.createNamedAsset("level-song").play(0, 65535);
      playMusic("menu");
      
      stage.addChild(kong);

      instructions = new Instructions(stage, true);
      instructions.addEventListener(TTApp.RETURN_TO_MENU, returnToMenu);

      backstory = new BackstoryChapter(stage, true);
      backstory.addEventListener(TTApp.RETURN_TO_MENU, returnToLevelSelector);
      
      mainMenu = new TimeTravelMainMenu(stage);

      mainMenu.addEventListener(TimeTravelMainMenu.SELECTION, mainMenuSelection);
      mainMenu.addToParent(this);
      mainMenu.makeActive();

      refreshLevelSelector();
    }

    public function mainMenuSelection(event:Event)
    {
      firstLevel = 1;
      
      if (mainMenu.selection == TimeTravelMainMenu.SELECTION_START_GAME || mainMenu.selection == TimeTravelMainMenu.SELECTION_CONTINUE)
      {
	if (mainMenu.selection == TimeTravelMainMenu.SELECTION_START_GAME)
	{
	  LevelManager.reset();
	  lm = LevelManager.getInstance();
	  mainMenu.setContinueEnabled(false);
	}

	refreshLevelSelector();
	
	prevChapter = mainMenu;
	nextChapter = LevelManager.backstoryViewed ? levelSelect : backstory;
	if (nextChapter == backstory) mainMenu.setContinueEnabled(true);
	var trans:ChapterTransition = new ChapterTransition(prevChapter, nextChapter, new SpriteFadeTransition(), 12);
	trans.addEventListener(ChapterTransition.COMPLETE_MESSAGE, done);
	trans.run();

	if (nextChapter != backstory) playMusic(null);
      }
      else if (mainMenu.selection == TimeTravelMainMenu.SELECTION_HOW_TO_PLAY)
      {
	prevChapter = mainMenu;
	nextChapter = instructions;
	var trans:ChapterTransition = new ChapterTransition(prevChapter, nextChapter, new SpriteFadeTransition(), 12);
	trans.addEventListener(ChapterTransition.COMPLETE_MESSAGE, done);
	trans.run();

	// playMusic(null);
      }
    }

    public function levelSelection(event:Event)
    {
      prevChapter = levelSelect;
      currentLevel = new TTApp(stage, levelSelect.selectedLevel);
      currentLevel.addEventListener(TTApp.RETURN_TO_MENU, returnToLevelSelector);
      currentLevel.addEventListener(TTApp.SUCCESSFUL_COMPLETION, markLevelComplete);
      nextChapter = currentLevel;
      var trans:ChapterTransition = new ChapterTransition(prevChapter, nextChapter, new SpriteFadeTransition(), 12);
      trans.addEventListener(ChapterTransition.COMPLETE_MESSAGE, done);
      trans.run();

      playMusic("level");
    }
    
    function returnToMenu(event:Event)
    {
      prevChapter = nextChapter;
      nextChapter = mainMenu;
      var trans:ChapterTransition = new ChapterTransition(prevChapter, nextChapter, new SpriteFadeTransition(), 12);
      trans.addEventListener(ChapterTransition.COMPLETE_MESSAGE, done);
      trans.run();
      
      playMusic("menu");
    }

    function markLevelComplete(event:Event)
    {
      lm.setLevelComplete(currentLevel.levelNumber, true);
      mainMenu.setContinueEnabled(true);
    }

    private function refreshLevelSelector()
    {
      if (levelSelect != null)
      {
	levelSelect.removeEventListener(TimeTravelMainMenu.SELECTION, levelSelection);
	levelSelect.removeEventListener(TTApp.RETURN_TO_MENU, returnToMenu);
	levelSelect.removeEventListener("next-levels", nextLevels);
	levelSelect.removeEventListener("previous-levels", previousLevels);
      }
      levelSelect = new LevelSelector(stage, firstLevel);
      lm.setLevelSelector(levelSelect);
      levelSelect.addEventListener(TimeTravelMainMenu.SELECTION, levelSelection);
      levelSelect.addEventListener(TTApp.RETURN_TO_MENU, returnToMenu);
      levelSelect.addEventListener("next-levels", nextLevels);
      levelSelect.addEventListener("previous-levels", previousLevels);
    }
    
    function returnToLevelSelector(event:Event)
    {
      refreshLevelSelector();

      prevChapter = nextChapter;
      nextChapter = levelSelect;
      var trans:ChapterTransition = new ChapterTransition(prevChapter, nextChapter, new SpriteFadeTransition(), 12);
      trans.addEventListener(ChapterTransition.COMPLETE_MESSAGE, done);
      trans.run();

      playMusic(null);
    }

    function nextLevels(event:Event)
    {
      prevChapter = levelSelect;
      
      firstLevel += 6;
      refreshLevelSelector();

      nextChapter = levelSelect;
      var trans:ChapterTransition = new ChapterTransition(prevChapter, nextChapter, new SpriteFadeTransition(), 12);
      trans.addEventListener(ChapterTransition.COMPLETE_MESSAGE, done);
      trans.run();

      playMusic(null);
    }

    function previousLevels(event:Event)
    {
      prevChapter = levelSelect;
      
      firstLevel -= 6;
      refreshLevelSelector();

      nextChapter = levelSelect;
      var trans:ChapterTransition = new ChapterTransition(prevChapter, nextChapter, new SpriteFadeTransition(), 12);
      trans.addEventListener(ChapterTransition.COMPLETE_MESSAGE, done);
      trans.run();

      playMusic(null);
    }
    
    function done(event:Event)
    {
      nextChapter.makeActive();
    }
  }
}

