package UserInterface
{
  import flash.display.Shape;
  import flash.display.SimpleButton;
  import flash.display.DisplayObject;
  import flash.display.Sprite;
  import flash.filters.*;
  import flash.text.TextField;
  import flash.text.TextFormat;
  import flash.text.TextFormatAlign;
  import flash.text.TextFieldAutoSize;
  import flash.text.TextFieldType;
  
  public class EasyButton extends SimpleButton {
    [Embed(systemFont='Phoenix Sans',
	   fontName='dakota',
	   mimeType='application/x-font')]
    private var dakotaFontClass:Class;
    
    private var label_str:String;
    private var fontSize:int;
    private var fontColor:uint;
    private var disabledFontColor:uint;
    private var shadowDepth:int;
    
    public function EasyButton(labelText:String = "Submit", fontSize:int = 24, fontColor:uint = 0x000000, shadowDepth = 8, disabledFontColor = 0x444444)
    {
      label_str = labelText;
      this.fontSize = fontSize;
      this.fontColor = fontColor;
      this.disabledFontColor = disabledFontColor;
      this.shadowDepth = shadowDepth;
      useHandCursor = true;
      upState = createUpState();
      overState = createOverState();
      downState = createDownState();
      hitTestState = createUpState();
    }

    public function setEnabled(value:Boolean)
    {
      if (value == true)
      {
	enabled = true;
	upState = createUpState(true);
	overState = createOverState(true);
      }
      else
      {
	enabled = false;
	upState = createUpState(false);
	overState = createOverState(false);
      }
    }
    
    private function createUpState(isEnabled:Boolean = true):Sprite
    {
      var state:Sprite = new Sprite();
      var upShape:Shape = new Shape();
      // upShape.graphics.lineStyle(2, 0xAAAAAA);
      // upShape.graphics.beginFill(0xEFEFEF);
      // upShape.graphics.drawRoundRect(0,0, 80,22, 10,10);
      var label_txt:TextField = createLabel(isEnabled);
      positionLabel(label_txt, upShape);
      state.addChild(upShape);
      state.addChild(label_txt);
      state.filters = [ new DropShadowFilter(shadowDepth, 45, 0x000000, 0.25) ];
      return state;
    }
    
    private function createOverState(isEnabled:Boolean = true):Sprite
    {
      var overShape:Shape = new Shape();
      // overShape.graphics.lineStyle(2, 0xAAAAAA);
      // overShape.graphics.beginFill(0xEFEFEF);
      // overShape.graphics.drawRoundRect(0,0, 80,22, 10,10);
      var label_txt:TextField = createLabel(isEnabled);
      positionLabel(label_txt, overShape);
      var state:Sprite = new Sprite();
      state.addChild(overShape);
      state.addChild(label_txt);
      state.filters = [ new GlowFilter(0xFF0000, 0.5, 12, 12), new DropShadowFilter(shadowDepth, 45, 0x000000, 0.25) ];
      return state;
    }
    
    private function createDownState():Sprite
    {
      var downShape:Shape = new Shape();
      // downShape.graphics.lineStyle(2, 0xAAAAAA);
      // downShape.graphics.beginFill(0xEFEFEF);
      // downShape.graphics.drawRoundRect(4,4, 80,22, 10,10);
      var label_txt:TextField = createLabel();
      positionLabel(label_txt, downShape, shadowDepth/2, shadowDepth/2);
      var state:Sprite = new Sprite();
      state.addChild(downShape);
      state.addChild(label_txt);
      state.filters = [ new GlowFilter(0xFF0000, 0.5, 12, 12), new DropShadowFilter(shadowDepth/2, 45, 0x000000, 0.25) ];
      return state;
    }
    
    private function createLabel(isEnabled:Boolean = true):TextField
    {
      var label_txt:TextField = new TextField();
      label_txt.autoSize = TextFieldAutoSize.CENTER;
      label_txt.selectable = false;
      label_txt.text = label_str;
      label_txt.background = false;
      var format:TextFormat = label_txt.getTextFormat();
      format.font = new dakotaFontClass().fontName; // "_sans";
      format.align = TextFormatAlign.CENTER;
      format.size = fontSize;
      format.color = isEnabled ? fontColor : disabledFontColor;
      label_txt.setTextFormat(format);
      label_txt.defaultTextFormat = format;
      label_txt.embedFonts = true;
      return label_txt;
    }
    
    private function positionLabel(label:TextField, state:DisplayObject, dx:int=0, dy:int=0):void
    {
      label.x = state.width/2 - label.width/2 + dx;
      label.y = state.height/2 - label.height/2 + dy;
    }
  }
}

