
package TimeTravel
{
  import flash.display.*;
  import flash.events.*;
  import flash.ui.*;

  public class Menu extends Sprite
  {
    var clip:MovieClip;
    var buttons:Array = [];
    var targetFrames:Array = [];
    var numButtons:int = 0;
    var selectedButton:int = -1;
    var oldState:Array = [];

    private function setSelection(ix:int)
    {
      if (selectedButton>=0) buttons[selectedButton].upState = oldState[selectedButton];
      selectedButton = ix;
      if (ix<0) return;
      oldState[ix] = buttons[ix].upState;
      buttons[ix].upState = buttons[ix].overState;
    }

    private function getButtonIndex(button:SimpleButton):int
    {
      for (var i:int=0; i<numButtons; ++i)
      {
        if (buttons[i] == button) return i;
      }
      return -1;
    }

    public function addEntry(button:SimpleButton, targetFrame:int)
    {
      if (getButtonIndex(button) >= 0) return;
      this.buttons.push(button);
      ++this.numButtons;
      this.targetFrames.push(targetFrame);
      this.oldState.push(null);
      button.addEventListener(MouseEvent.CLICK, this.clickHandler);
      button.addEventListener(MouseEvent.MOUSE_OVER, this.mouseOverHandler);
      button.addEventListener(MouseEvent.MOUSE_OUT, this.mouseOutHandler);
    }

    public function clickHandler(event:Event)
    {
      var ix:int = getButtonIndex(SimpleButton(event.currentTarget));
      if (ix>=0) trace("jumping to frame: " + targetFrames[ix]);
      else trace("clickHandler failed... couldn't find button");
    }

    public function mouseOverHandler(event:Event)
    {
      var ix:int = getButtonIndex(SimpleButton(event.currentTarget));
      if (ix>=0) setSelection(ix);
    }

    public function mouseOutHandler(event:Event)
    {
      var ix:int = getButtonIndex(SimpleButton(event.currentTarget));
      if (ix>=0) setSelection(-1);
    }

    public function keyHandler(event:KeyboardEvent)
    {
      if (numButtons == 0) return;
      trace("menu got keypress" + event.keyCode)
      if (event.keyCode == Keyboard.DOWN || event.keyCode == Keyboard.RIGHT)
      {
        if (event.type == KeyboardEvent.KEY_DOWN) setSelection((selectedButton + 1)% numButtons);
      }
      else if (event.keyCode == Keyboard.UP || event.keyCode == Keyboard.LEFT)
      {
        setSelection((selectedButton - 1 + numButtons) % numButtons);
      }
      else if (event.keyCode == Keyboard.SPACE || event.keyCode == Keyboard.ENTER)
      {
        // trace("pressed enter; selected button is " + selectedButton);
        if (event.type == KeyboardEvent.KEY_DOWN)
          buttons[selectedButton].dispatchEvent(new MouseEvent(MouseEvent.MOUSE_DOWN));
        else {
          buttons[selectedButton].dispatchEvent(new MouseEvent(MouseEvent.MOUSE_UP));
          buttons[selectedButton].dispatchEvent(new MouseEvent(MouseEvent.CLICK));
        }
      }  
    }

    public function Menu(clip:MovieClip)
    {
      this.clip = clip;
      clip.stage.addEventListener(KeyboardEvent.KEY_DOWN, this.keyHandler);
      clip.stage.addEventListener(KeyboardEvent.KEY_UP, this.keyHandler);
    }
  }
}

